"""Virtual Dwelling Generator - Generates a virtual dwelling stock
"""
import numpy as np
from energy_demand.basic import lookup_tables

from energy_demand.technologies import diffusion_technologies


class Dwelling(object):
    """Dwelling or aggregated group of dwellings

    Arguments
    ----------
    curr_yr : int
        Current year of simulation
    coordinates : float
        coordinates
    dwtype : int
        Dwelling type id. Description can be found in `daytype_lu`
    house_id : int
        Unique ID of dwelling or dwelling group
    age : int
        Age of dwelling in years (year the building was built)
    pop : float
        Dwelling population
    floorarea : float
        Floor area of dwelling
    hlc : float
        Heat loss coefficient
    hdd : float
        Heating degree days

    Note
    -----
    - Depending on service or residential model, not all attributes
      are filled (then they are inistialised as None or zero)
    - For every dwelling, the scenario drivers are calculated for each enduse
    """
    def __init__(
            self,
            curr_yr,
            coordinates,
            floorarea,
            enduses,
            driver_assumptions,
            population=None,
            age=None,
            dwtype=None,
            sector=None,
            gva=None
        ):
        """Constructor of Dwelling Class
        """
        self.curr_yr = curr_yr
        self.enduses = enduses
        self.longitude = coordinates['longitude']
        self.latitude = coordinates['longitude']
        self.dwtype = dwtype
        self.age = age
        self.population = population
        self.floorarea = floorarea
        self.sector = sector
        self.gva = gva

        #: Calculate heat loss coefficient with age and dwelling type if possible
        self.hlc = get_hlc(dwtype, age)

        # Generate attribute for each enduse containing calculated scenario driver value
        self.calc_scenario_driver(driver_assumptions)

        # Testing
        assert floorarea != 0

    def calc_scenario_driver(self, driver_assumptions):
        """Sum scenario drivers per enduse and add as attribute

        Arguments
        ---------
        driver_assumptions : dict
            Scenario drivers for every enduse
        """
        for enduse in self.enduses:

            #used to sum (not zero!)
            scenario_driver_value = 1

            # If there is no scenario drivers for enduse, set to standard value 1
            if enduse not in driver_assumptions:
                Dwelling.__setattr__(self, enduse, scenario_driver_value)
            else:
                scenario_drivers = driver_assumptions[enduse]

                # Iterate scenario driver and get attriute to multiply values
                try:
                    for scenario_driver in scenario_drivers:

                        # If scenario driver is set to zero, do not use this driver
                        driver_value = getattr(self, scenario_driver)

                        # Ignore zero driver values
                        if driver_value == 0:
                            pass
                        else:
                            scenario_driver_value *= driver_value
                except TypeError:
                    #logging.info("Scenario driver `%s` calculation not possible", scenario_driver)
                    pass

                Dwelling.add_new_attribute(
                    self,
                    enduse,
                    scenario_driver_value)

            assert scenario_driver_value != 0

    def add_new_attribute(self, name, value):
        """Add a new self asttribute to DwellingStock
        """
        setattr(self, name, value)

class DwellingStock(object):
    """Class of the building stock in a region
    """
    def __init__(self, dwellings, enduses):
        """Returns a new building stock object for every `region`.

        Arguments
        ----------
        dwellings : list
            List containing all dwelling objects
        enduses : list
            Enduses
        """
        self.dwellings = dwellings

        # Calculate pop of dwelling stock
        self.population = get_tot_pop(dwellings)

        # Calculate enduse specific scenario driver
        for enduse in enduses:

            enduse_scenario_driver = self.get_scenario_driver(
                enduse)

            DwellingStock.add_new_attribute(
                self,
                enduse,
                enduse_scenario_driver)

    def get_scenario_driver(self, enduse):
        """Sum all scenario driver for an enduse

        Arguments
        ----------
        enduse: string
            Enduse to calculate scenario drivers
        """
        sum_driver = 0
        for dwelling in self.dwellings:
            sum_driver += getattr(dwelling, enduse)

        return sum_driver

    def add_new_attribute(self, name, value):
        """Add a new self asttribute to DwellingStock
        """
        setattr(self, name, value)

def get_tot_pop(dwellings):
    """Get total population of all dwellings

    Return
    ------
    tot_pop : float or bool
        If population is not provided, return `None`,
        otherwise summed population of all dwellings
    """
    tot_pop = 0

    for dwelling in dwellings:
        if dwelling.population is None:
            return None
        else:
            tot_pop += dwelling.population

    return tot_pop

def get_floorare_pp(
        floorarea,
        reg_pop_by,
        base_yr,
        sim_period,
        assump_diff_floorarea_pp
    ):
    """Calculate future floor area per person depending
    on assumptions on final change and base year data

    Arguments
    ----------
    floorarea : dict
        Floor area base year for all regions
    reg_pop_by : dict
        Population of base year for all regions
    base_yr, : int
        base year
    sim_period: list
        Simulation period
    assump_diff_floorarea_pp : float
        Assumption of change in floor area up to end of simulation

    Returns
    -------
    floor_area_pp : dict
        Contains all values for floor area per person for every year

    Note
    ----
    - Linear change of floor area per person is assumed over time
    """
    floor_area_pp = {}

    if reg_pop_by == 0:
        floor_area_pp[base_yr] = 0
    else:
        # Floor area per person of base year
        floor_area_pp[base_yr] = floorarea / reg_pop_by

    for curr_yr in sim_period:
        if curr_yr == base_yr:
            pass
        else:
            # Floor area of current year = floor area of base year * change
            floor_area_pp[curr_yr] = floor_area_pp[base_yr] * (1 + assump_diff_floorarea_pp[curr_yr])

    return floor_area_pp

def get_dwtype_floor_area(
        dwtype_floorarea_by,
        dwtype_floorarea_future,
        base_yr,
        sim_period
    ):
    """Calculates the floor area per dwelling type for every year

    Arguments
    ----------
    dwtype_distr_by : dict
        Distribution of dwelling types base year
    dwtype_floorarea_future : dict
        Distribution of future dwelling types end year
    base_yr : list
        Simulation parameters
    sim_period : list
        Simulation period
    sim_period_yrs : list
        Nr of simlated years

    Returns
    -------
    dwtype_floor_area : dict
        Contains the floor area change per dwelling type
    Note
    -----
    - A linear change over time is assumed

    Example
    -------
    out = {year: {'dwtype': 0.3}}
    """
    dwtype_floor_area = {}

    # Base year
    dwtype_floor_area[base_yr] = dwtype_floorarea_by

    # Simulation years
    for curr_yr in sim_period:

        if curr_yr == base_yr:
            pass
        else:
            y_distr = {}

            for dwtype in dwtype_floorarea_by:
                val_by = dwtype_floorarea_by[dwtype]
                val_future = dwtype_floorarea_future[dwtype]

                yr_until_changed = dwtype_floorarea_future['yr_until_changed']

                val_cy = diffusion_technologies.linear_diff(
                    base_yr,
                    curr_yr,
                    val_by,
                    val_future,
                    yr_until_changed)

                y_distr[dwtype] = val_cy

            dwtype_floor_area[curr_yr] = y_distr

    return dwtype_floor_area

def get_dwtype_distr(
        dwtype_distr_by,
        dwtype_distr_fy,
        base_yr,
        sim_period
    ):
    """Calculates the annual distribution of dwelling types
    based on assumption of base and end year distribution

    Arguments
    ----------
    dwtype_distr_by : dict
        Distribution of dwelling types base year
    dwtype_distr_fy : dict
        Distribution of dwelling types end year

    Returns
    -------
    dwtype_distr : dict
        Contains all dwelling type distribution for every year
    Note
    -----
    - A linear change over time is assumed

    Example
    -------
    out = {year: {'dwtype': 0.3}}
    """
    dwtype_distr = {}

    # Base year
    dwtype_distr[base_yr] = dwtype_distr_by

    # Simulation years
    for curr_yr in sim_period:
        if curr_yr == base_yr:
            pass
        else:
            y_distr = {}

            for dwtype in dwtype_distr_by:
                val_by = dwtype_distr_by[dwtype]
                val_future = dwtype_distr_fy[dwtype]

                yr_until_changed = dwtype_distr_fy['yr_until_changed']

                val_cy = diffusion_technologies.linear_diff(
                    base_yr,
                    curr_yr,
                    val_by,
                    val_future,
                    yr_until_changed)

                y_distr[dwtype] = val_cy

            dwtype_distr[curr_yr] = y_distr

    # Test if distribution is 100%
    for year in dwtype_distr:
        np.testing.assert_almost_equal(
            sum(dwtype_distr[year].values()),
            1.0,
            decimal=5,
            err_msg='The distribution of dwelling types went wrong', verbose=True)

    return dwtype_distr

def ss_dw_stock(
        region,
        enduses,
        sectors,
        scenario_data,
        reg_coord,
        assumptions,
        curr_yr,
        base_yr,
        virtual_building_stock_criteria
    ):
    """Create dwelling stock for service sector

    Arguments
    ----------
    regions : dict
        Regions
    data : dict
        Data container

    Returns
    -------
    dwelling_stock : list
        List with objects

    Note
    ----
    - Iterate years and change floor area depending on assumption on
      linear change up to ey
    """
    dw_stock = []
    for sector in sectors:

        # If virtual building stock, change floor area proportionally to population
        if virtual_building_stock_criteria:
            pop_by = scenario_data['population'][base_yr][region]
            pop_cy = scenario_data['population'][curr_yr][region]

            pop_factor = pop_cy / pop_by
            lin_diff_factor = pop_factor
            '''
            # Change in floor are for service submodel depending on sector
            # (if no change set to 1, if e.g. 10% decrease change to 0.9)
            assumptions.ss_floorarea_change_ey_p = {

                'yr_until_changed': yr_until_changed_all_things,

                'community_arts_leisure': 1,
                'education': 1,
                'emergency_services': 1,
                'health': 1,
                'hospitality': 1,
                'military': 1,
                'offices': 1,
                'retail': 1,
                'storage': 1,
                'other': 1}
                '''
        else:
            # Change in floor area up to end year
            if sector in assumptions.ss_floorarea_change_ey_p:
                change_floorarea_p_ey = assumptions.ss_floorarea_change_ey_p[sector]
                yr_until_changed = assumptions.ss_floorarea_change_ey_p['yr_until_changed']
            else:
                raise Exception(
                    "Error: The ss building stock sector floor area assumption is not defined")

            # Floor area of sector in current year considering linear diffusion
            lin_diff_factor = diffusion_technologies.linear_diff(
                base_yr,
                curr_yr,
                1.0,
                change_floorarea_p_ey,
                yr_until_changed)

        floorarea_sector_by = scenario_data['floor_area']['ss_floorarea'][base_yr][region][sector]
        floorarea_sector_cy = floorarea_sector_by * lin_diff_factor

        # GVA data
        try:
            gva_sector_lu = lookup_tables.economic_sectors_regional_MISTRAL()
            gva_nr = gva_sector_lu[sector]['match_int']
            gva_dw_data = scenario_data['gva_industry'][curr_yr][region][gva_nr]
        except KeyError:
            # If not sector specific GVA, use overal GVA per head
            gva_dw_data = scenario_data['gva_per_head'][curr_yr][region]

        # Create dwelling objects
        dw_stock.append(
            Dwelling(
                curr_yr=curr_yr,
                coordinates=reg_coord[region],
                population=pop_cy,
                floorarea=floorarea_sector_cy,
                enduses=enduses,
                driver_assumptions=assumptions.scenario_drivers,
                sector=sector,
                gva=gva_dw_data))

    # Add regional base year dwelling to dwelling stock
    dwelling_stock = DwellingStock(
        dw_stock,
        enduses)

    return dwelling_stock

def rs_dw_stock(
        region,
        assumptions,
        scenario_data,
        sim_yrs,
        dwelling_types,
        enduses,
        reg_coord,
        driver_assumptions,
        curr_yr,
        base_yr,
        virtual_building_stock_criteria
    ):
    """Creates a virtual building stock for every year and region

    Arguments
    ----------
    region : dict
        Region name
    curr_yr : int
        Current year

    Returns
    -------
    dwelling_stock : dict
        Building stock wei

    reg_dw_stock_by : Base year building stock
        reg_building_stock_yr : Building stock for every simulation year

    Notes
    -----
    - The assumption about internal temperature change is
      used as for each dwelling the hdd are calculated
      based on wheater data and assumption on t_base

    - Doesn't take floor area as an input but calculates floor area
      based on floor area pp parameter. However, floor area
      could be read in by:

      1.) Inserting `tot_floorarea_cy = data['rs_floorarea'][curr_yr]`

      2.) Replacing 'dwtype_floor_area', 'dwtype_distr' and 'data_floorarea_pp'
          with more specific information from real building stock model
    """
    if virtual_building_stock_criteria:

        # Get changes in absolute floor area per dwelling type over time
        dwtype_floor_area = get_dwtype_floor_area(
            assumptions.dwtype_floorarea_by,
            assumptions.dwtype_floorarea_fy,
            base_yr,
            sim_yrs)

        # Get distribution of dwelling types of all simulation years
        dwtype_distr = get_dwtype_distr(
            assumptions.dwtype_distr_by,
            assumptions.dwtype_distr_fy,
            base_yr,
            sim_yrs)

        # Get floor area per person for every simulation year
        data_floorarea_pp = get_floorare_pp(
            scenario_data['floor_area']['rs_floorarea'][base_yr][region],
            scenario_data['population'][base_yr][region],
            base_yr,
            sim_yrs,
            assumptions.non_regional_vars['assump_diff_floorarea_pp'])

        # Get fraction of total floorarea for every dwelling type
        floorarea_p = get_floorarea_dwtype_p(
            dwelling_types,
            dwtype_floor_area,
            dwtype_distr)

        floorarea_by = scenario_data['floor_area']['rs_floorarea'][base_yr][region]

        population_by = scenario_data['population'][base_yr][region]
        population_cy = scenario_data['population'][curr_yr][region]

        if population_by != 0:
            floorarea_pp_by = floorarea_by / population_by # [m2 / person]
        else:
            floorarea_pp_by = 0

        # Calculate new necessary floor area  per person of current year
        floorarea_pp_cy = data_floorarea_pp[curr_yr]

        # Calculate new floor area
        tot_floorarea_cy = floorarea_pp_cy * population_cy
    else:
        """
        #If floor_area is read in from model, this would be here
        # NEeds to be implemented
        # NEWCASTLE
        """
        floorarea_by = scenario_data['floor_area']['rs_floorarea'][curr_yr][region]
        tot_floorarea_cy = scenario_data['floor_area']['rs_floorarea'][curr_yr][region]

    new_floorarea_cy = tot_floorarea_cy - floorarea_by

    # Only calculate changing
    if curr_yr == base_yr:
        dw_stock_base = generate_dw_existing(
            driver_assumptions=driver_assumptions,
            scenario_data=scenario_data,
            enduses=enduses,
            reg_coord=reg_coord,
            region=region,
            curr_yr=curr_yr,
            dw_lu=dwelling_types,
            floorarea_p=floorarea_p[base_yr],
            floorarea_by=floorarea_by,
            dwtype_age_distr_by=assumptions.dwtype_age_distr[base_yr],
            floorarea_pp=floorarea_pp_by,
            gva_dw_data=scenario_data['gva_per_head'][curr_yr][region])

        # Create regional base year building stock
        dwelling_stock = DwellingStock(
            dw_stock_base,
            enduses)
    else:
        """The number of people in the base year dwelling stock may change.
        If the floor area pp decreased with constant pop, the same number of
        people will be living in too large houses. It is not assumed
        that area is demolished.
        """
        floor_area_cy = floorarea_pp_cy * population_by

        if floor_area_cy > floorarea_by:
            demolished_area = 0
        else:
            demolished_area = floorarea_by - floor_area_cy

        remaining_area = floorarea_by - demolished_area

        # Generate stock for existing area
        dw_stock_cy = generate_dw_existing(
            driver_assumptions=driver_assumptions,
            scenario_data=scenario_data,
            enduses=enduses,
            reg_coord=reg_coord,
            region=region,
            curr_yr=curr_yr,
            dw_lu=dwelling_types,
            floorarea_p=floorarea_p[curr_yr],
            floorarea_by=remaining_area,
            dwtype_age_distr_by=assumptions.dwtype_age_distr[base_yr],
            floorarea_pp=floorarea_pp_cy,
            gva_dw_data=scenario_data['gva_per_head'][curr_yr][region])

        # Append buildings of new floor area to
        if new_floorarea_cy > 0:
            dw_stock_cy = generate_dw_new(
                driver_assumptions=driver_assumptions,
                scenario_data=scenario_data,
                reg_coord=reg_coord,
                enduses=enduses,
                dwtypes=dwelling_types,
                region=region,
                curr_yr=curr_yr,
                floorarea_p_by=floorarea_p[curr_yr],
                floorarea_pp_cy=floorarea_pp_cy,
                dw_stock_new_dw=dw_stock_cy,
                new_floorarea_cy=new_floorarea_cy,
                gva_dw_data=scenario_data['gva_per_head'][curr_yr][region])
        else:
            pass # no new floor area is added

        # Generate region and save it in dictionary (Add old and new buildings to stock)
        dwelling_stock = DwellingStock(
            dw_stock_cy,
            enduses)

    return dwelling_stock

def get_floorarea_dwtype_p(dw_lookup, dw_floorarea, dwtype_distr):
    """Calculates the percentage of the total floor area
    belonging to each dwelling type. Depending on average
    floor area per dwelling type and the dwelling type
    distribution, the percentages are calculated
    for ever simulation year

    Arguments
    ----------
    dw_lookup : dw_lookup
        Dwelling types
    dw_floorarea : dict
        Floor area per type and year
    dwtype_distr : dict
        Distribution of dwelling type over the simulation period

    Returns
    -------
    dw_floorarea_p : dict
        Contains the percentage of the total floor
        area for each dwtype for every simulation year (must be 1.0 in tot)

    Notes
    -----
    This calculation is necessary as the share of dwelling types may differ depending the year
    """
    dw_floorarea_p = {}

    for curr_yr, type_distr_p in dwtype_distr.items():
        area_dw_type = {}

        # Calculate share of dwelling area based on absolute size and distribution
        for dw_type in dw_lookup.values():
            # Get absolut size of dw_type
            area_dw_type[dw_type] = type_distr_p[dw_type] * dw_floorarea[curr_yr][dw_type]

        # Convert absolute values into percentages
        tot_area = sum(area_dw_type.values())
        for dw_type, dw_type_area in area_dw_type.items():
            area_dw_type[dw_type] = dw_type_area / tot_area

        dw_floorarea_p[curr_yr] = area_dw_type

    return dw_floorarea_p

def generate_dw_existing(
        driver_assumptions,
        scenario_data,
        enduses,
        reg_coord,
        region,
        curr_yr,
        dw_lu,
        floorarea_p,
        floorarea_by,
        dwtype_age_distr_by,
        floorarea_pp,
        gva_dw_data
    ):
    """Generates dwellings according to age, floor area
    and distribution assumption

    Arguments
    ----------
    assumptions : dict
        Assumptions
    scenario_data : dict
        Scenario data
    enduses : list
        Enduses
    region : dict
        Region name
    curr_yr : int
        Base year
    dw_lu : dict
        Dwelling type look-up
    floorarea_p : dict
        Fraction of floor area per dwelling type
    floorarea_by : dict
        Floor area of base year
    dwtype_age_distr_by : dict
        Age distribution of dwelling
    floorarea_pp : dict
        Floor area per person
    tot_floorarea_cy : float
        Floor are in current year
    pop_by : dict
        Population in base year

    Return
    ------
    dw_stock_by : list
        Dwelling stocks in a list
    """
    dw_stock_by, control_pop, control_floorarea = [], 0, 0

    # Iterate dwelling types
    for dwtype_name in dw_lu.values():

        # Calculate floor area per dwelling type
        dwtype_floorarea = floorarea_p[dwtype_name] * floorarea_by

        # Distribute according to age
        for dwtype_age, distribution in dwtype_age_distr_by.items():

            # Floor area of dwelling_class_age (distribute proportionally floor area)
            dwtype_age_class_floorarea = dwtype_floorarea * distribution

            # Floor area per person is divided by base area value to calc pop
            if floorarea_pp != 0:
                pop_dwtype_age_class = dwtype_age_class_floorarea / floorarea_pp
            else:
                pop_dwtype_age_class = 0

            # create building object
            dw_stock_by.append(
                Dwelling(
                    curr_yr=curr_yr,
                    coordinates=reg_coord[region],
                    floorarea=dwtype_age_class_floorarea,
                    enduses=enduses,
                    driver_assumptions=driver_assumptions,
                    population=pop_dwtype_age_class,
                    age=float(dwtype_age),
                    dwtype=dwtype_name,
                    gva=gva_dw_data))

            control_floorarea += dwtype_age_class_floorarea
            control_pop += pop_dwtype_age_class

    return dw_stock_by

def generate_dw_new(
        driver_assumptions,
        scenario_data,
        reg_coord,
        enduses,
        dwtypes,
        region,
        curr_yr,
        floorarea_p_by,
        floorarea_pp_cy,
        dw_stock_new_dw,
        new_floorarea_cy,
        gva_dw_data
    ):
    """Generate dwelling objects for all new dwellings

    All new dwellings are appended to the existing
    building stock of the region

    Arguments
    ----------
    data : dict
        Data container
    region : str
        Region
    curr_yr : int
        Current year
    floorarea_p_by : dict
        Fraction of floorarea in base year
    floorarea_pp_cy : dict
        Floor area per person in current year
    dw_stock_new_dw : dict
        New dwellings
    new_floorarea_cy : dict
        New floorarea in current year

    Returns
    -------
    dw_stock_new_dw : list
        List with appended dwellings

    Notes
    -----
    The floor area id divided proprtionally depending on dwelling type
    Then the population is distributed
    builindg is creatd
    """
    control_pop, control_floorarea = 0, 0

    # Iterate dwelling types
    for dwtype_name in dwtypes.values():

        # Calculate new floor area per dewlling type
        dw_type_new_floorarea = floorarea_p_by[dwtype_name] * new_floorarea_cy

        # Calculate pop (Floor area is divided by floorarea_per_person)
        pop_dwtype_new_build_cy = dw_type_new_floorarea / floorarea_pp_cy

        # create building object
        dw_stock_new_dw.append(
            Dwelling(
                curr_yr=curr_yr,
                coordinates=reg_coord[region],
                floorarea=dw_type_new_floorarea,
                enduses=enduses,
                driver_assumptions=driver_assumptions,
                population=pop_dwtype_new_build_cy,
                age=curr_yr,
                dwtype=dwtype_name,
                gva=gva_dw_data))

        control_floorarea += dw_type_new_floorarea
        control_pop += pop_dwtype_new_build_cy

    # Test if floor area and pop are the same
    #assert round(new_floorarea_cy, 3) == round(control_floorarea, 3)
    #assert round(new_floorarea_cy/floorarea_pp_cy, 2) == round(control_pop, 2)
    return dw_stock_new_dw

def get_hlc(dw_type, age):
    """Calculates the linearly derived heat loss coeeficients
    depending on age and dwelling type

    Arguments
    ----------
    dw_type : int
        Dwelling type
    age : int
        Age of dwelling

    Returns
    -------
    hls : Heat loss coefficient [W/m2 * K]

    Notes
    -----
    Source: Linear trends derived from Table 3.17 ECUK Tables
    https://www.gov.uk/government/collections/energy-consumption-in-the-uk
    """
    if dw_type is None or age is None:
        #logging.debug(
        #    "The HLC could not be calculated of a dwelling age: {} dw_type: {}".format(dw_type, age))
        return None
    else:
        # Dict with linear fits for all different dwelling types {dw_type: [slope, constant]}
        linear_fits_hlc = {
            'detached': [-0.0223, 48.292],
            'semi_detached': [-0.0223, 48.251],
            'terraced': [-0.0223, 48.063],
            'flat': [-0.0223, 47.02],
            'bungalow': [-0.0223, 48.261]}

        # Get linearly fitted value
        hlc = linear_fits_hlc[dw_type][0] * age + linear_fits_hlc[dw_type][1]

        return hlc

'''def createNEWCASTLE_dwelling_stock(curr_yr, region, data, parameter_list):
    """Create dwelling stock based on input from
    building model from Newcastle

    Arguments
    ---------
    curr_yr : int
        Current year
    """
    # ------
    # Get all regional information concering floor area etc.
    # Either preprocessed or direct
    # ------
    stock_pop = {}
    floor_area = {}
    # Iterate all buildings
    # see which dwelling type -->
    # see which age_class -->
    # see which building attribute (e.g. resid, industry)
    # --> summen the following attributes according these categories
    #       - population stock_pop[]
    #       - floor area
    #       -
    # ----------------
    # Create residential dwelling stock
    # ----------------
    # Create dwellings Residential
    # Inputs Residential
    #   - building_type/dwelling_type/dwellingtype_ageclass
    #   -
    dw_stock = []
    dwelling_types = ["detached", "semi_detached"]
    age_classs = [1920, 1930, 1940] #Age categories
    building_type = "residential"

    for dwelling_type in dwelling_types:
        for age_class in age_classs:

            # Get pop of age class
            pop_dwtype_age_class = stock_pop[building_type][dwelling_type][age_class]

            # Get floor area of building type, dwelling_ype, age_class
            floor_area_dwtype_age_class = floor_area[building_type][dwelling_type][age_class]

            dwelling_obj = Dwelling(
                curr_yr=curr_yr,
                coordinates=data['reg_coord'][region],
                floorarea=floor_area_dwtype_age_class,
                enduses=data['enduses']['residential'],
                driver_assumptions=data['assumptions'].scenario_drivers['rs_submodule'],
                population=pop_dwtype_age_class,
                age=age_class,
                dwtype=dwelling_type,
                gva=data['scenario_data']['gva'][curr_yr][region])
            dw_stock.append(dwelling_obj)

    dwelling_stock = DwellingStock(
        dw_stock,
        data['enduses']['residential'])

    return dwelling_stock
'''


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import

import pkg_resources

try:
    __version__ = pkg_resources.get_distribution(__name__).version
except:
    __version__ = 'unknown'


"""
"""
import numpy as np
import matplotlib.pyplot as plt

from energy_demand.plotting import basic_plot_functions
from energy_demand import enduse_func
#from energy_demand.plotting import plotting_styles

def plt_regions_peak_h(
        results_every_year,
        lookups,
        regions,
        path_plot_fig,
        fueltype_str_to_plot
    ):
    """Plot
    Arguments
    ---------
    """
    fig = plt.figure(
        figsize=basic_plot_functions.cm2inch(14, 8))

    ax = fig.add_subplot(1, 1, 1)

    nr_y_to_plot = len(results_every_year)

    legend_entries = []

    for fueltype_str, fueltype in lookups['fueltypes'].items():

        if fueltype_str != fueltype_str_to_plot:
            pass
        else:
            # Legend
            # Read out fueltype specific load
            data_over_years = {}
            for reg_nr, reg_geocode in enumerate(regions):
                data_over_years[reg_nr] = []

            for model_year_object in results_every_year.values():

                for reg_nr, reg_geocode in enumerate(regions):

                    # Get peak hour value
                    _, peak_fueltyp_h = enduse_func.get_peak_day_single_fueltype(model_year_object[fueltype][reg_nr])

                    # Add peak hour
                    data_over_years[reg_nr].append(peak_fueltyp_h)
    
            y_init = data_over_years

    # ----------
    # Plot lines
    # ----------
    #linestyles = plotting_styles.linestyles()
    years = list(results_every_year.keys())

    for reg in y_init:
        plt.plot(
            years,
            y_init[reg],
            #linestyle=linestyles[fueltype],
            color='lightblue',
            linewidth=0.2,)

    ax.legend(
        legend_entries,
        prop={
            'family': 'arial',
            'size': 8},
        frameon=False)

    # -
    # Axis
    # -
    base_yr = 2015
    major_interval = 10
    minor_interval = 5

    # Major ticks
    major_ticks = np.arange(base_yr, years[-1] + major_interval, major_interval)
    ax.set_xticks(major_ticks)

    # Minor ticks
    minor_ticks = np.arange(base_yr, years[-1] + minor_interval, minor_interval)
    ax.set_xticks(minor_ticks, minor=True)

    plt.xlim(2015, years[-1])

    # --------
    # Labeling
    # --------
    plt.ylabel("GW")
    plt.xlabel("year")
    plt.title("ED peak hour, y, all enduses, single regs")

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    # Save fig
    fig.savefig(path_plot_fig)
    plt.close()



"""Function related to service or fuel switch
"""
import logging
from collections import defaultdict
from energy_demand.technologies import tech_related
from energy_demand.read_write import read_data
from energy_demand.basic import basic_functions
from energy_demand.scripts import init_scripts

def get_all_narrative_points(switches, enduse):
    """Get all narrative points of an enduse

    Arguments
    ---------
    switches : list
        All switches
    enduse : str
        Enduse

    Returns
    -------
    switch_yrs : list
        All sorted defined timesteps from narrative
    """
    switch_yrs = set([])

    for switch in switches:
        if switch.enduse == enduse:
            switch_yrs.add(switch.switch_yr)

    list(switch_yrs).sort()

    return switch_yrs

def get_switch_criteria(
        enduse,
        sector,
        crit_switch_happening,
        base_yr=False,
        curr_yr=False
    ):
    """Test if switch is happending
    """
    if base_yr == curr_yr and (base_yr != False or curr_yr != False):
        crit_switch_service = False
    else:
        if enduse in crit_switch_happening:

            # If None, then switch is true across all sectors or sector == None
            if not crit_switch_happening[enduse] or not sector:
                crit_switch_service = True
            else:
                if sector in crit_switch_happening[enduse]:
                    crit_switch_service = True
                else:
                    crit_switch_service = False
        else:
            crit_switch_service = False

    return crit_switch_service

def sum_fuel_across_sectors(fuels):
    """Sum fuel across sectors of an enduse if multiple sectors.
    Otherwise return unchanged `fuels`

    Arguments
    ---------
    fuels : dict or np.array
        Fuels of an enduse either for sectors or already aggregated

    Returns
    -------
    sum_array : np.array
        Sum of fuels of all sectors
    """
    if isinstance(fuels, dict):
        sum_array = sum(fuels.values())
        return sum_array
    else:
        return fuels

def get_share_s_tech_ey(
        service_switches,
    ):
    """Get fraction of service for each technology
    defined in a switch for every narrative year

    Arguments
    ---------
    service_switches : list
        Service switches

    Return
    ------
    enduse_tech_ey_p : dict
        Enduse, share per technology in ey
    """
    narrative_yrs_enduse_tech_ey_p = defaultdict(dict)

    # Calculate enduse tech_ey_p for every narrative temporal point
    for region, switches in service_switches.items():

        # Get all enduses defined in switches
        enduses = get_all_enduses_of_switches(switches)

        for enduse in enduses:
            narrative_yrs_enduse_tech_ey_p[enduse][region] = defaultdict(dict)

            for switch in switches:
                if switch.enduse == enduse:
                    narrative_yrs_enduse_tech_ey_p[enduse][region][switch.switch_yr][switch.technology_install] = switch.service_share_ey

    return dict(narrative_yrs_enduse_tech_ey_p)

def create_switches_from_s_shares(
        enduse,
        s_tech_by_p,
        switch_technologies,
        tech_enduse_by,
        enduse_switches,
        s_tot_defined,
        sector,
        switch_yr
    ):
    """
    Create swiches from service shares.
    If not 100% of the service share is defined,
    the switches get defined which proportionally
    decrease service of all not switched technologies

    Arguments
    ----------
    enduse : str
        Enduse
    s_tech_by_p :
        Service share per technology in base year
    switch_technologies : list
        Technologies involved in switch
    tech_enduse_by : list
        Technologies of enduse and sector
    enduse_switches : list
        Switches
    s_tot_defined : float
        Service defined
    sector : str
        Sector
    switch_yr : int
        Year

    Returns
    -------
    service_switches_out : dict
        Service switches
    """
    service_switches_out = []

    # Calculate relative by proportion of not assigned technologies in base year
    tech_not_assigned_by_p = {}

    for tech in tech_enduse_by:
        if tech not in switch_technologies:
            tech_not_assigned_by_p[tech] = s_tech_by_p[sector][enduse][tech]

    # Normalise: convert to percentage
    tot_share_not_assigned = sum(tech_not_assigned_by_p.values())

    # If no fuel is assigned to enduse, do not change switches
    if tot_share_not_assigned == 0:

        # Add zero share for all technologies not assigned if sums up to 100%
        zero_share = 0
        for tech in tech_not_assigned_by_p:
            switch_new = read_data.ServiceSwitch(
                enduse=enduse,
                sector=sector,
                technology_install=tech,
                service_share_ey=zero_share,
                switch_yr=switch_yr)
            enduse_switches.append(switch_new)

        service_switches_out = enduse_switches
    else:
        for tech, share_by in tech_not_assigned_by_p.items():
            tech_not_assigned_by_p[tech] = share_by / tot_share_not_assigned

        # Get all defined technologies in base year
        for tech in tech_enduse_by:
            if tech not in switch_technologies:
                if s_tot_defined == 1.0:
                    # All technologies are fully defined
                    switch_new = read_data.ServiceSwitch(
                        enduse=enduse,
                        sector=sector,
                        technology_install=tech,
                        service_share_ey=0,
                        switch_yr=switch_yr)
                else:
                    # Calculate not defined share in switches
                    s_not_assigned = 1 - s_tot_defined

                    # Reduce not assigned share proportionally
                    tech_ey_p = tech_not_assigned_by_p[tech] * s_not_assigned

                    switch_new = read_data.ServiceSwitch(
                        enduse=enduse,
                        sector=sector,
                        technology_install=tech,
                        service_share_ey=tech_ey_p,
                        switch_yr=switch_yr)

                service_switches_out.append(switch_new)
            else:
                # If assigned copy to final switches
                for switch_new in enduse_switches:
                    if switch_new.technology_install == tech:

                        # If no fuel demand, also no switch possible
                        if tot_share_not_assigned == 0:
                            switch_new.service_share_ey = 0

                        service_switches_out.append(switch_new)

    return service_switches_out

def get_all_enduses_of_switches(switches):
    '''Read all endueses of defined switches

    Arguments
    ---------
    switches : list
        Defined switches

    Returns
    -------
    enduses : list
        All enduses defined in switches
    '''
    enduses = set([])
    for switch in switches:
        enduses.add(switch.enduse)
    enduses = list(enduses)

    return enduses

def get_all_sectors_of_switches(switches, enduse_to_match):
    '''

    Arguments
    ---------
    switches : list
        Defined switches

    Returns
    -------
    enduses : list
        All enduses defined in switches
    '''
    sectors = set([])
    for switch in switches:
        if switch.enduse == enduse_to_match:
            sectors.add(switch.sector)

    sectors = list(sectors)

    return sectors

def get_sectors_of_enduse(enduse_to_match, enduses, sectors):
    """Get corresponding sectors of an enduse

    Arguments
    ---------
    enduse_to_match : str
        Enduse to match sectors
    enduses : dict
        All enduses per submodel
    sectors : dict
        All sectors per submodel

    Returns
    -------
    sectors_of_enduse : list
        All sectors of an enduse
    """
    for submodel, submodel_enduses in enduses.items():
        if enduse_to_match in submodel_enduses:
            sectors_of_enduse = sectors[submodel]

    return sectors_of_enduse

def autocomplete_switches(
        service_switches,
        specified_tech_enduse_by,
        s_tech_by_p,
        enduses,
        sectors,
        crit_all_the_same=True,
        regions=False,
        f_diffusion=False,
        techs_affected_spatial_f=False,
        service_switches_from_capacity=[]
    ):
    """Add not defined technologies in switches and set
    correct future service share.

    If the defined service switches do not sum up to 100% service,
    the remaining service is distriputed proportionally
    to all remaining technologies.

    Arguments
    ---------
    service_switches : dict
        Defined service switches
    specified_tech_enduse_by : dict
        Specified technologies of an enduse
    s_tech_by_p : dict
        Share of service of technology in base year
    enduses : list
        Enduses
    sectors : list
        Sectors
    crit_all_the_same : bool
        Criteria wheter regionally specific or not
    regions : list
        Regions
    f_diffusion :
        Diffusion factors
    techs_affected_spatial_f : list
        Technologies affected by spatial diffusion
    service_switches_from_capacity : list
        Transformed capacity swithces into service switches

    Returns
    -------
    reg_share_s_tech_ey_p : dict
        Shares per technology in end year
    service_switches_out : list
        Added services switches which now in total sum up to 100%
    """
    reg_share_s_tech_ey_p = {}
    service_switches_out = {}

    # Enduses form direct switches
    switch_enduses = get_all_enduses_of_switches(service_switches)

    # Enduses from capacity witches
    any_region = list(service_switches_from_capacity.keys())[0]
    switch_enduses_capacity_switches = get_all_enduses_of_switches(
        service_switches_from_capacity[any_region])

    # All enduses
    switch_enduses = set(switch_enduses + switch_enduses_capacity_switches)

    # Iterate enduses
    for enduse in switch_enduses:
        logging.debug("... calculating service switches: {} crit_all_the_same: {}".format(enduse, crit_all_the_same))

        sectors_of_enduse = get_sectors_of_enduse(
            enduse, enduses, sectors)

        for sector in sectors_of_enduse:

            switches_enduse = init_scripts.get_sector__enduse_switches(
                sector, enduse, service_switches)

            # -----------------------------------------
            #  Append service switches which were
            #  generated from capacity switches
            # -----------------------------------------
            service_switches_out[sector] = {}
            for region in regions:
                service_switches_out[sector][region] = service_switches_from_capacity[region]

            if crit_all_the_same:
                updated_switches = []

                # Get all narrative years
                switch_yrs = get_all_narrative_points(
                    switches_enduse, enduse=enduse)

                # Execut function for every narrative year
                for switch_yr in switch_yrs:
                    enduse_switches = []
                    s_tot_defined = 0
                    switch_technologies = []

                    for switch in switches_enduse:
                        if switch.enduse == enduse and switch.switch_yr == switch_yr:
                            s_tot_defined += switch.service_share_ey
                            switch_technologies.append(switch.technology_install)
                            enduse_switches.append(switch)

                    # Calculate relative by proportion of not assigned technologies
                    switches_new = create_switches_from_s_shares(
                        enduse=enduse,
                        s_tech_by_p=s_tech_by_p,
                        switch_technologies=switch_technologies,
                        tech_enduse_by=specified_tech_enduse_by[enduse][sector],
                        enduse_switches=enduse_switches,
                        s_tot_defined=s_tot_defined,
                        sector=sector,
                        switch_yr=switch_yr)

                    updated_switches.extend(switches_new)

                # For every region, set same switches
                for region in regions:
                    service_switches_out[sector][region].extend(updated_switches)
            else:
                for region in regions:

                    # Get all narrative year
                    switch_yrs = get_all_narrative_points(
                        service_switches, enduse=enduse)

                    # Execut function for yvery narrative year
                    for switch_yr in switch_yrs:

                        # Get all switches of this enduse
                        enduse_switches = []
                        s_tot_defined = 0
                        switch_technologies = []

                        for switch in switches_enduse:
                            if switch.enduse == enduse and switch.switch_yr == switch_yr:

                                # Global share of technology diffusion
                                s_share_ey_global = switch.service_share_ey

                                # If technology is affected by spatial exlicit diffusion
                                if switch.technology_install in techs_affected_spatial_f:

                                    # Regional diffusion calculation
                                    s_share_ey_regional = s_share_ey_global * f_diffusion[enduse][region]

                                    # -------------------------------------
                                    # if larger than max crit, set to 1
                                    # -------------------------------------
                                    max_crit = 1
                                    if s_share_ey_regional > max_crit:
                                        s_share_ey_regional = max_crit

                                    if s_tot_defined + s_share_ey_regional > 1.0:

                                        if round(s_tot_defined + s_share_ey_regional) > 1:
                                            # TODO TODO TODO
                                            logging.warning(
                                                "{}  {} {}".format(s_tot_defined, s_share_ey_regional, s_tot_defined + s_share_ey_regional))
                                            raise Exception(
                                                "Error of regional parameter calcuation. More than one technology switched with larger share") 
                                        else:
                                            s_share_ey_regional = max_crit - s_share_ey_regional
                                else:
                                    s_share_ey_regional = switch.service_share_ey

                                switch_new = read_data.ServiceSwitch(
                                    enduse=switch.enduse,
                                    sector=switch.sector,
                                    technology_install=switch.technology_install,
                                    service_share_ey=s_share_ey_regional,
                                    switch_yr=switch.switch_yr)

                                s_tot_defined += s_share_ey_regional
                                switch_technologies.append(switch.technology_install)
                                enduse_switches.append(switch_new)

                                # Create switch
                                switches_new = create_switches_from_s_shares(
                                    enduse=enduse,
                                    s_tech_by_p=s_tech_by_p,
                                    switch_technologies=switch_technologies,
                                    specified_tech_enduse_by=specified_tech_enduse_by,
                                    enduse_switches=enduse_switches,
                                    s_tot_defined=s_tot_defined,
                                    sector=sector,
                                    switch_yr=switch_yr)

                                service_switches_out[sector][region].extend(switches_new)

            # Calculate fraction of service for each technology
            reg_share_s_tech_ey_p[sector] = get_share_s_tech_ey(
                service_switches_out[sector])

    return reg_share_s_tech_ey_p

def capacity_switch(
        narrative_timesteps,
        regions,
        capacity_switches,
        technologies,
        fuels,
        fuel_shares_enduse_by,
        base_yr
    ):
    """Create service switches based on assumption on
    changes in installed fuel capacity (`capacity switches`).
    Service switch are calculated based on the assumed
    capacity installation (in absolute GW) of technologies.
    Assumptions on capacities are defined in the
    CSV file `switches_capacity.csv`

    Note
    -----
    With the information about the installed capacity
    only the change in percentage in end year gets calulated.
    This means that there is not absolute change in demand (
        e.g. you cant add additional demand of an existing technology
        --> It is only a switch)
    Arguments
    ---------
    narrative_timesteps : dict
        List with narrative timestep for every enduse
    regions : list
        Regions
    capacity_switches : list
        Capacity switches
    technologies : dict
        Technologies
    fuels : dict
        Fuels
    fuel_shares_enduse_by : dict
        Fuel technology shares in base year
    base_yr : int
        Base year

    Returns
    -------
    service_switches : dict
        Updated service switches containing converted
        capacity switches to service switches
    """
    service_switches = {}

    for region in regions:
        service_switches[region] = []

        # Get all affected enduses of capacity switches
        switch_enduses = get_all_enduses_of_switches(
            capacity_switches[region])

        if switch_enduses == []:
            pass # no capacity switch defined
        else:
            for enduse in switch_enduses:

                # Get all capacity switches related to this enduse
                enduse_capacity_switches = get_switches_of_enduse(
                    capacity_switches[region], enduse, crit_region=False)

                for switch in enduse_capacity_switches:

                    sector_info_crit = basic_functions.test_if_sector(
                        fuel_shares_enduse_by[enduse])

                    # Check if enduse has sectors
                    if not sector_info_crit:
                        fuel_shares = fuel_shares_enduse_by[enduse]
                    else:
                        any_sector = list(fuel_shares_enduse_by[enduse].keys())[0]
                        fuel_shares = fuel_shares_enduse_by[enduse][any_sector]

                    # Convert capacity switches to service switches
                    enduse_service_switches = create_service_switch(
                        narrative_timesteps[enduse],
                        enduse,
                        switch.sector,
                        switch,
                        enduse_capacity_switches,
                        technologies,
                        fuel_shares,
                        base_yr,
                        fuels[enduse])

                    # Add service switches
                    service_switches[region] += enduse_service_switches

    return service_switches

def create_service_switch(
        narrative_timesteps,
        enduse,
        sector,
        switch,
        enduse_capacity_switches,
        technologies,
        fuel_shares_enduse_by,
        base_yr,
        fuel_enduse_y
    ):
    """Generate service switch based on capacity assumptions

    Arguments
    ---------
    narrative_timesteps : list
        Narrative timesteps
    enduse : dict
        Enduse
    switch : obj
        Capacity switch
    enduse_capacity_switches : list
        All capacity switches of an enduse (see warning)
    technologies : dict
        Technologies
    fuel_shares_enduse_by : dict
        Fuel shares per enduse for base year
    base_yr : int
        base year
    fuel_enduse_y : dict
        Fuels

    Returns
    ------
    service_switches : dict
        Service switches

    Major steps
    -------
        1.  Convert fuel per technology to service in ey
        2.  Convert installed capacity to service of ey and add service
        3.  Calculate percentage of service for ey
        4.  Write out as service switch
    """
    service_switches_enduse = []

    for switch_yr in narrative_timesteps:

        # Get switches of narrative timestep
        capacity_switches = get_switches_of_enduse(
            enduse_capacity_switches, enduse, year=switch_yr, crit_region=False)

        # ---------------------------------------------
        # Calculate service per technology for switch_yr
        # ---------------------------------------------
        service_enduse_tech = {}

        for fueltype, tech_fuel_shares in fuel_shares_enduse_by.items():
            for tech, fuel_share_by in tech_fuel_shares.items():

                # Efficiency of year when capacity is fully installed
                eff_cy = tech_related.calc_eff_cy(
                    base_yr,
                    switch.switch_yr,
                    technologies[tech].eff_by,
                    technologies[tech].eff_ey,
                    technologies[tech].year_eff_ey,
                    technologies[tech].eff_achieved,
                    technologies[tech].diff_method)

                # Convert to service (fuel * fuelshare * eff)
                s_tech_ey_y = fuel_enduse_y[fueltype] * fuel_share_by * eff_cy
                service_enduse_tech[tech] = s_tech_ey_y

        # -------------------------------------------
        # Calculate service of installed capacity of
        # increased (installed) technologies
        # -------------------------------------------
        for switch in capacity_switches:
            eff_cy = tech_related.calc_eff_cy(
                base_yr,
                switch.switch_yr,
                technologies[switch.technology_install].eff_by,
                technologies[switch.technology_install].eff_ey,
                technologies[switch.technology_install].year_eff_ey,
                technologies[switch.technology_install].eff_achieved,
                technologies[switch.technology_install].diff_method)

            # Convert installed capacity to service
            installed_capacity_ey = switch.installed_capacity * eff_cy

            # Add capacity
            service_enduse_tech[switch.technology_install] += installed_capacity_ey

        # -------------------------------------------
        # Calculate service in % per enduse
        # -------------------------------------------
        tot_s = sum(service_enduse_tech.values())
        for tech, service_tech in service_enduse_tech.items():
            service_enduse_tech[tech] = service_tech / tot_s

        # -------------------------------------------
        # Add to switch of technology_install
        # -------------------------------------------
        for tech, s_tech_p in service_enduse_tech.items():

            service_switch = read_data.ServiceSwitch(
                enduse=enduse,
                sector=sector,
                technology_install=tech,
                service_share_ey=s_tech_p,
                switch_yr=switch_yr)

            service_switches_enduse.append(service_switch)

    return service_switches_enduse

def get_switches_of_enduse(
        switches,
        enduse,
        year=False,
        crit_region=True
    ):
    """Get all fuel switches of a specific enduse and
    year (optional)

    Arguments
    ----------
    switches : list
        Switches
    enduse : str
        Enduse
    year : int,default=False
        Year
    crit_region : bool, default=True
        Criteria whether region specific switches

    Returns
    -------
    enduse_switches : list
        All switches of a specific enduse
    """
    if crit_region:
        enduse_switches = {}
        for reg in switches:
            enduse_switches[reg] = []
            for switch in switches[reg]:

                if not year:
                    if switch.enduse == enduse:
                        enduse_switches[reg].append(switch)
                else:
                    if switch.enduse == enduse and switch.switch_yr == year:
                        enduse_switches[reg].append(switch)
    else:
        enduse_switches = []
        for switch in switches:
            if not year:
                if switch.enduse == enduse:
                    enduse_switches.append(switch)
            else:
                if switch.enduse == enduse and switch.switch_yr == year:
                    enduse_switches.append(switch)

    return enduse_switches

def switches_to_dict(service_switches):
    """Write switch to dict, i.e. providing service fraction
    of technology as dict: {tech: service_ey_p}

    Arguments
    ---------
    service_switches : dict
        Service switches

    Returns
    -------
    s_tech_by_p : dict
        Service tech after service switch

    service_switches : dict
        Reg, fuel_swtiches
    """
    s_tech_by_p = defaultdict(dict)

    for reg, reg_switches in service_switches.items():
        for switch in reg_switches:
            s_tech_by_p[reg][switch.technology_install] = switch.service_share_ey

        assert round(sum(s_tech_by_p[reg].values()), 3) == 1

    return dict(s_tech_by_p)



"""Function to generate plots based on simulation results stored in a folder
"""
import os
import sys

from energy_demand.result_processing import single_scenario

def process_result_multi_scen(
        path_to_folder_with_scenarios="C:/Users/cenv0553/ed/results/Fig_11",
        path_shapefile_input="C:/Users/cenv0553/ED/data/region_definitions/lad_2016_uk_simplified.shp",
        base_yr=2015,
        comparison_year=2050
    ):
    """Iterate the folders with scenario
    runs and generate PDF results of individual
    simulation runs

    Arguments
    ----------
    path_to_folder_with_scenarios : str
        Path to folders with stored results
    """

    # Chose which plots should be generated
    plot_crit_dict = {
        "spatial_results": True,              # Spatial geopanda maps

        "plot_differences_p": True,           # Spatial maps of percentage difference per fueltype over time
        "plot_total_demand_fueltype": True, #False,  # Spatial maps of total demand per fueltype over time
        "plot_population": True,             # Spatial maps of population
        "plot_load_factors": True,           # Spatial maps of load factor
        "plot_load_factors_p": True,         # Spatial maps of load factor change
        "plot_abs_peak_h": True,             # Spatial maps of peak h demand
        "plot_diff_peak_h": True,             # Spatial maps of peak h difference (%)
        "plot_stacked_enduses": True,
        "plot_y_all_enduses": True,
        "plot_fuels_enduses_y": True,
        "plot_lf": True,
        "plot_week_h": True,
        "plot_h_peak_fueltypes": True,
        "plot_averaged_season_fueltype": True, # Compare for every season and daytype the daily loads
        "plot_radar": True,
        "plot_radar_seasonal": True,                      # Plot radar spider charts
        "plot_line_for_every_region_of_peak_demand": True,
        "plot_lad_cross_graphs": True}

    # Get all folders with scenario run results (name of folder is scenario)
    scenarios = os.listdir(path_to_folder_with_scenarios)

    scenario_names_ignored = [
        '__results_multiple_scenarios',
        '_FigII_non_regional_2015',
        '_results_PDF_figs']

    for scenario in scenarios:
        print("... scenario: " + str(scenario))
        if scenario in scenario_names_ignored:
            pass
        else:
            single_scenario.main(
                os.path.join(path_to_folder_with_scenarios, scenario),
                path_shapefile_input,
                plot_crit_dict,
                base_yr=base_yr,
                comparison_year=comparison_year)
    return

if __name__ == '__main__':
    # Map command line arguments to function arguments.
    process_result_multi_scen(*sys.argv[1:])




"""Script to run after installation of HIRE

Script function which are executed after model installation and
"""
import os
import zipfile
import configparser

from energy_demand.basic import basic_functions
from energy_demand.assumptions import general_assumptions
from energy_demand.scripts import s_rs_raw_shapes
from energy_demand.scripts import s_ss_raw_shapes
from energy_demand.read_write import data_loader
from energy_demand.basic import lookup_tables
from energy_demand.scripts.smif_data_related import script_data_preparation_MISTRAL_pop_gva


def post_install_setup(args):
    """Run this function after installing the energy_demand
    model with smif and putting the data folder with all necessary
    data into a local drive. This scripts only needs to be
    executed once after the energy_demand model has been installed

    Arguments
    ----------
    args : object
        Arguments defined in ``./cli/__init__.py``
    """
    print("... start running initialisation scripts", flush=True)

    path_config_file = args.config_file


    config = data_loader.read_config_file(path_config_file)

    local_data_path = config['PATHS']['path_local_data']

    base_yr = config['CONFIG']['base_yr']

    data = {}
    data['paths'] = config['CONFIG_DATA']
    data['local_paths'] = config['DATA_PATHS']
    data['result_paths'] = basic_functions.get_result_paths(config['PATHS']['path_result_data'])
    data['lookups'] = lookup_tables.basic_lookups()
    data['enduses'], data['sectors'], data['fuels'], lookup_enduses, \
        lookup_sector_enduses = data_loader.load_fuels(data['paths'])

    data['assumptions'] = general_assumptions.Assumptions(
        lookup_enduses=lookup_enduses,
        lookup_sector_enduses=lookup_sector_enduses,
        base_yr=base_yr,
        paths=data['paths'],
        enduses=data['enduses'],
        sectors=data['sectors'])

    # Delete all previous data from previous model runs
    basic_functions.del_previous_setup(data['local_paths']['data_processed'])
    basic_functions.del_previous_setup(data['result_paths']['data_results'])
    basic_functions.del_previous_setup(data['local_paths']['path_post_installation_data'])

    # Create folders and subfolder for data_processed
    folders_to_create = [
        data['local_paths']['data_processed'],
        data['local_paths']['path_post_installation_data'],
        data['local_paths']['load_profiles'],
        data['local_paths']['rs_load_profile_txt'],
        data['local_paths']['ss_load_profile_txt']]

    for folder in folders_to_create:
        basic_functions.create_folder(folder)

    print("... Read in residential submodel load profiles", flush=True)
    '''s_rs_raw_shapes.run(
        data['paths'],
        data['local_paths'],
        base_yr)

    print("... Read in service submodel load profiles", flush=True)
    s_ss_raw_shapes.run(
        data['paths'],
        data['local_paths'],
        data['lookups'])

    # Input data preparation
    print("Generate additional data", flush=True)'''

    # Extract NISMOD population data
    path_to_zip_file = os.path.join(local_data_path,"population-economic-smif-csv-from-nismod-db.zip")
    path_extraction = os.path.join(local_data_path, 'scenarios', "MISTRAL_pop_gva")
    zip_ref = zipfile.ZipFile(path_to_zip_file, 'r')
    zip_ref.extractall(path_extraction)
    zip_ref.close()

    # Complete gva and pop data for every sector
    data_pop = os.path.join(local_data_path, "scenarios", "MISTRAL_pop_gva", "data")
    path_geography = os.path.join(local_data_path, "scenarios", "uk_pop_principal_2015_2050_MSOA_england.csv")
    geography_name = "lad_uk_2016"

    # All MISTRAL scenarios to prepare with correct config
    scenarios_to_generate = ['pop-baseline16_econ-c16_fuel-c16']
    #scenarios_to_generate = []

    script_data_preparation_MISTRAL_pop_gva.run(
        path_to_folder=data_pop,
        path_MSOA_baseline=path_geography,
        MSOA_calculations=False,
        geography_name=geography_name,
        scenarios_to_generate=scenarios_to_generate)

    print("... successfully finished setup")
    return



"""Run HIRE locally outside the SMIF framework

Note
----
Always execute from root folder and pass the path to the .ini file as argument:

    Examples
    python energy_demand/energy_demand/main.py C:/Users/cenv0553/ed/energy_demand/local_run_config_file.ini test_run
    python energy_demand/energy_demand/main.py C:/Users/cenv0553/ed/energy_demand/local_run_config_file.ini h_max NF1 h_max

"""
import os
import sys
import time
import logging
from collections import defaultdict
import pandas as pd
import numpy as np

from energy_demand.basic import basic_functions, date_prop, demand_supply_interaction, testing_functions, lookup_tables
from energy_demand import model
from energy_demand.assumptions import strategy_vars_def, general_assumptions
from energy_demand.read_write import read_data, write_data, data_loader
from energy_demand.validation import lad_validation
from energy_demand.scripts import s_scenario_param, init_scripts, s_disaggregation
from energy_demand.plotting import fig_enduse_yh
from energy_demand import enduse_func
from energy_demand.technologies import tech_related

def energy_demand_model(
        regions,
        data,
        criterias,
        assumptions,
        weather_stations,
        weather_yr,
        weather_by
    ):
    """Main function of energy demand model to calculate yearly demand

    Arguments
    ----------
    regions : list
        Regions
    data : dict
        Data container
    assumptions : dict
        Assumptions
    weather_yr: int
        Year of weather data

    Returns
    -------
    result_dict : dict
        A nested dictionary containing all data for energy supply model with
        timesteps for every hour in a year.
        [fueltype : region : timestep]
    modelrun_obj : dict
        Object of a yearly model run

    Note
    ----
    This function is executed in the wrapper
    """
    logging.info("... Number of modelled regions: %s", len(regions))

    modelrun = model.EnergyDemandModel(
        regions=regions,
        data=data,
        criterias=criterias,
        assumptions=assumptions,
        weather_stations=weather_stations,
        weather_yr=weather_yr,
        weather_by=weather_by)

    # Calculate base year demand
    fuels_in = testing_functions.test_function_fuel_sum(
        data, data['fuel_disagg'], criterias['mode_constrained'], assumptions.enduse_space_heating)

    # Log model results
    write_data.logg_info(modelrun, fuels_in, data)

    return modelrun

if __name__ == "__main__":
    """
    """
    # Update cluster file: #os.path.dirname(__file__), '..', 'local_run_config_file_cluster.ini'))
    if os.path.isfile(sys.argv[1]):
        path_config  = sys.argv[1]
    else:
        raise Exception("The defined .ini file does not exist_ {}".format(sys.argv[1]))
    config = data_loader.read_config_file(path_config)

    data = {}
    #sim_yrs = [2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050]
    sim_yrs = [2015, 2030, 2050]

    if len(sys.argv) > 3: #user defined arguments are provide
        print("Arguments taken from comand line")
        scenario_name = str(sys.argv[2])        # Scenario name
        weather_realisation = str(sys.argv[3])  # Weather realisation
        name_config_path = str(sys.argv[4])     # Config path
    else:
        print("Default Arguments used")
        scenario_name = str(sys.argv[2])
        weather_realisation = 'NF1'
        name_config_path = str(sys.argv[2])

    print("-------------------------------------")
    print("Information")
    print("-------------------------------------")
    print("local_data_path             " + str(sys.argv[1]))
    print("Configuration path:         " + str(path_config))
    print("Simulated yrs               " + str(sim_yrs))
    print("name_config_path:           " + str(name_config_path))
    print("weather_realisation:        " + str(weather_realisation))

    # Local path configurations
    if config['CRITERIA']['cluster_calc']:
        path_weather_data = "/soge-home/staff/cenv0553/_weather_realisation"
    else:
        path_weather_data = "C:/Users/cenv0553/ED/data/scenarios"

    if name_config_path == 'h_max' or name_config_path == 'l_max' or name_config_path in ['test_run', 'h_max_0', 'h_max_10', 'h_max_0_only', 'h_max_10_only']:
        local_scenario = 'pop-b_econ-c_fuel-c'
    elif name_config_path == 'h_min' or name_config_path == 'l_min' or name_config_path in ['h_min_zero', 'h_min_5dm', 'h_min_10dm']:
        local_scenario = 'pop-f_econ-c_fuel-c'

    # --- Paths
    path_strategy_vars = os.path.join(config['DATA_PATHS']['path_strategy_vars'], name_config_path)
    name_region_set = os.path.join(config['PATHS']['path_local_data'], 'region_definitions', "lad_2016_uk_simplified.shp")
    name_population_dataset = os.path.join(config['PATHS']['path_local_data'], 'scenarios', 'MISTRAL_pop_gva', 'data', '{}/population__lad.csv'.format(local_scenario))
    name_gva_dataset = os.path.join(config['PATHS']['path_local_data'], 'scenarios', 'MISTRAL_pop_gva', 'data', '{}/gva_per_head__lad_sector.csv'.format(local_scenario))
    name_gva_dataset_per_head = os.path.join(config['PATHS']['path_local_data'],'scenarios', 'MISTRAL_pop_gva', 'data', '{}/gva_per_head__lad.csv'.format(local_scenario))

    # --------------------
    # Create scenario path
    # --------------------
    name_scenario_run = "{}_result_local_{}".format(scenario_name, str(time.ctime()).replace(":", "_").replace(" ", "_"))
    path_new_scenario = os.path.join(config['PATHS']['path_result_data'], name_scenario_run)

    # -----------------------------------------------------------------------
    # Create new folders
    # -----------------------------------------------------------------------
    basic_functions.del_previous_setup(path_new_scenario)
    basic_functions.create_folder(path_new_scenario)

    # --------------------
    # Load all other paths
    # --------------------
    data['paths'] = config['CONFIG_DATA']
    data['local_paths'] = config['DATA_PATHS']
    data['result_paths'] = basic_functions.get_result_paths(path_new_scenario)

    folders_to_create = [
        data['result_paths']['data_results'],
        data['result_paths']['data_results_model_run_pop'],
        data['result_paths']['data_results_validation']]
    for folder in folders_to_create:
        basic_functions.create_folder(folder)

    # ----------------------------------------------------------------------
    # Load data
    # ----------------------------------------------------------------------
    print("a " + str(name_region_set))
    data['scenario_data'] = defaultdict(dict)
    data['enduses'], data['sectors'], data['fuels'], lookup_enduses, lookup_sector_enduses = data_loader.load_fuels(data['paths'])
    data['regions'] = read_data.get_region_names(name_region_set)
    data['reg_coord'] = basic_functions.get_long_lat_decimal_degrees(read_data.get_region_centroids(name_region_set))
    print("AA " + str(name_gva_dataset))
    data['scenario_data']['population'] = data_loader.read_scenario_data(name_population_dataset, region_name='lad_uk_2016', value_name='population')
    data['scenario_data']['gva_industry'] = data_loader.read_scenario_data_gva(name_gva_dataset, region_name='lad_uk_2016', value_name='gva_per_head', all_dummy_data=False)
    data['scenario_data']['gva_per_head'] = data_loader.read_scenario_data(name_gva_dataset_per_head, region_name='lad_uk_2016', value_name='gva_per_head')

    # -----------------------------
    # Assumptions
    # -----------------------------
    data['assumptions'] = general_assumptions.Assumptions(
        lookup_enduses=lookup_enduses,
        lookup_sector_enduses=lookup_sector_enduses,
        base_yr=config['CONFIG']['base_yr'],
        weather_by=config['CONFIG']['user_defined_weather_by'],
        simulation_end_yr=config['CONFIG']['user_defined_simulation_end_yr'],
        curr_yr=2015,
        sim_yrs=sim_yrs,
        paths=data['paths'],
        enduses=data['enduses'],
        sectors=data['sectors'],
        reg_nrs=len(data['regions']))

    # -----------------------------------------------------------------------------
    # Calculate population density for base year
    # -----------------------------------------------------------------------------
    region_objects = read_data.get_region_objects(name_region_set)
    data['pop_density'] = {}
    for region in region_objects:
        region_name = region['properties']['name']
        region_area = region['properties']['st_areasha']
        data['pop_density'][region_name] = data['scenario_data']['population'][data['assumptions'].base_yr][region_name] / region_area

    # -----------------------------------------------------------------------------
    # Load standard strategy variable values from .py file
    # Containing full information
    # -----------------------------------------------------------------------------        
    default_streategy_vars = strategy_vars_def.load_param_assump(
        default_values=True)

    # -----------------------------------------------------------------------------
    # Load standard smif parameters and generate standard single timestep narrative for simulation end year
    # -----------------------------------------------------------------------------
    strategy_vars = strategy_vars_def.generate_default_parameter_narratives(
        default_streategy_vars=default_streategy_vars,
        end_yr=2050,
        base_yr=config['CONFIG']['base_yr'])

    # -----------------------------------------
    # User defines stragey variable from csv files
    # -----------------------------------------
    user_defined_vars = data_loader.load_local_user_defined_vars(
        default_strategy_var=default_streategy_vars,
        path_csv=path_strategy_vars,
        simulation_base_yr=data['assumptions'].base_yr,
        simulation_end_yr=config['CONFIG']['user_defined_simulation_end_yr'])

    strategy_vars = data_loader.replace_variable(user_defined_vars, strategy_vars)

    # Replace strategy variables not defined in csv files
    strategy_vars_out = strategy_vars_def.autocomplete_strategy_vars(
        strategy_vars,
        narrative_crit=True)

    data['assumptions'].update('strategy_vars', strategy_vars_out)

    # -----------------------------------------------------------------------------
    # Load necessary data
    # -------------------------------------------------------------------------------
    data['tech_lp'] = data_loader.load_data_profiles(
        data['paths'], data['local_paths'],
        data['assumptions'].model_yeardays,
        data['assumptions'].model_yeardays_daytype,)

    technologies = general_assumptions.update_technology_assumption(
        data['assumptions'].technologies,
        data['assumptions'].strategy_vars['f_eff_achieved'],
        data['assumptions'].strategy_vars['gshp_fraction'])
    data['assumptions'].technologies.update(technologies)

    if config['CRITERIA']['virtual_building_stock_criteria']:
        data['scenario_data']['floor_area']['rs_floorarea'], data['scenario_data']['floor_area']['ss_floorarea'], data['service_building_count'], rs_regions_without_floorarea, ss_regions_without_floorarea = data_loader.floor_area_virtual_dw(
            data['regions'],
            data['sectors'],
            data['local_paths'],
            data['scenario_data']['population'][data['assumptions'].base_yr],
            data['assumptions'].base_yr)

        # Add all areas with no floor area data
        data['assumptions'].update("rs_regions_without_floorarea", rs_regions_without_floorarea)
        data['assumptions'].update("ss_regions_without_floorarea", ss_regions_without_floorarea)

    print("Start Energy Demand Model with python version: " + str(sys.version))
    print("-----------------------------------------------")
    print("Number of Regions                        " + str(data['assumptions'].reg_nrs))

    # Obtain population data for disaggregation
    if config['CRITERIA']['msoa_crit']:
        name_population_dataset = data['local_paths']['path_population_data_for_disaggregation_MSOA']
    else:
        name_population_dataset = data['local_paths']['path_population_data_for_disaggregation_lad']
    data['pop_for_disag'] =  data_loader.read_scenario_data(name_population_dataset)

    # ---------------------------------------------
    # Make selection of weather stations and data
    # Load all temperature and weather station data
    # ---------------------------------------------
    data['weather_stations'], data['temp_data'] = data_loader.load_temp_data(
        data['local_paths'],
        sim_yrs=sim_yrs,
        weather_realisation=weather_realisation,
        path_weather_data=path_weather_data,
        same_base_year_weather=False,
        crit_temp_min_max=config['CRITERIA']['crit_temp_min_max'],
        load_np=False,
        load_parquet=False,
        load_csv=True)

    # ------------------------------------------------------------
    # Disaggregate national energy demand to regional demands
    # ------------------------------------------------------------
    data['fuel_disagg'] = s_disaggregation.disaggr_demand(
        data,
        crit_temp_min_max=config['CRITERIA']['crit_temp_min_max'],
        spatial_calibration=config['CRITERIA']['spatial_calibration'])

    # ------------------------------------------------------------
    # Calculate spatial diffusion factors
    # ------------------------------------------------------------
    real_values = data['pop_density']
    f_reg, f_reg_norm, f_reg_norm_abs, crit_all_the_same = init_scripts.create_spatial_diffusion_factors(
        narrative_spatial_explicit_diffusion=data['assumptions'].strategy_vars['spatial_explicit_diffusion'],
        fuel_disagg=data['fuel_disagg'],
        regions=data['regions'],
        real_values=real_values,
        narrative_speed_con_max=data['assumptions'].strategy_vars['speed_con_max'])

    print("Criteria all regions the same:           " + str(crit_all_the_same))

    # ------------------------------------------------
    # Calculate parameter values for every region
    # ------------------------------------------------
    regional_vars = init_scripts.spatial_explicit_modelling_strategy_vars(
        data['assumptions'].strategy_vars,
        data['assumptions'].spatially_modelled_vars,
        data['regions'],
        data['fuel_disagg'],
        f_reg,
        f_reg_norm,
        f_reg_norm_abs)
    data['assumptions'].update('strategy_vars', regional_vars)

    # -----------------------------------------------------------------
    # Calculate parameter values for every simulated year based on narratives
    # and add also general information containter for every parameter
    # -----------------------------------------------------------------
    print("... starting calculating values for every year")
    regional_vars, non_regional_vars = s_scenario_param.generate_annual_param_vals(
        data['regions'],
        data['assumptions'].strategy_vars,
        sim_yrs)

    # ------------------------------------------------
    # Calculate switches (not generic)
    # ------------------------------------------------
    try:
        service_switches_raw = pd.read_csv(os.path.join(path_strategy_vars, "switches_service.csv"))
        service_switches = read_data.service_switch(service_switches_raw)
    except:
        service_switches = []

    fuel_switches = read_data.read_fuel_switches(os.path.join(path_strategy_vars, "switches_fuel.csv"), data['enduses'], data['assumptions'].fueltypes, data['assumptions'].technologies)
    capacity_switches = read_data.read_capacity_switch(os.path.join(path_strategy_vars, "switches_capacity.csv"))

    # Load Generic fuel switches
    crit_switch_happening = testing_functions.switch_testing(
        fuel_switches=fuel_switches,
        service_switches=service_switches,
        capacity_switches=capacity_switches)
    data['assumptions'].update('crit_switch_happening', crit_switch_happening)

    print("... starting calculating switches")
    annual_tech_diff_params = init_scripts.switch_calculations(
        sim_yrs,
        data,
        f_reg,
        f_reg_norm,
        f_reg_norm_abs,
        crit_all_the_same,
        service_switches=service_switches,
        fuel_switches=fuel_switches,
        capacity_switches=capacity_switches)

    for region in data['regions']:
        regional_vars[region]['annual_tech_diff_params'] = annual_tech_diff_params[region]

    data['assumptions'].update('regional_vars', regional_vars)
    data['assumptions'].update('non_regional_vars', non_regional_vars)

    # ------------------------------------------------
    # Spatial Validation
    # ------------------------------------------------
    if config['CRITERIA']['validation_criteria'] == True and config['CRITERIA']['cluster_calc'] != True:
        lad_validation.spatial_validation_lad_level(
            data['fuel_disagg'],
            data['result_paths']['data_results_validation'],
            data['paths'],
            data['regions'],
            data['reg_coord'],
            config['CRITERIA']['plot_crit'])

    # -----------------------------------
    # Only selection of regions to simulate
    # -------------------------------------
    if config['CRITERIA']['reg_selection']:
        region_selection = read_data.get_region_selection(
            os.path.join(
                data['local_paths']['local_path_datafolder'],
                "region_definitions",
                config['CRITERIA']['reg_selection_csv_name']))
        #region_selection = ['E02003237', 'E02003238']

        setattr(data['assumptions'], 'reg_nrs', len(region_selection))
    else:
        region_selection = data['regions']

    # -------------------------------------------
    # Create .ini file with simulation information
    # -------------------------------------------
    write_data.write_simulation_inifile(
        path_new_scenario,
        data,
        region_selection)

    # Write population data to file
    for sim_yr in data['assumptions'].sim_yrs:
        write_data.write_scenaric_population_data(
            sim_yr,
            os.path.join(path_new_scenario, 'model_run_pop'),
            list(data['scenario_data']['population'][sim_yr].values()))

    # -----------------------
    # Main model run function
    # -----------------------
    for sim_yr in data['assumptions'].sim_yrs:

        print("Local simulation for year:  " + str(sim_yr))
        setattr(data['assumptions'], 'curr_yr', sim_yr) # Set current year

        if config['CRITERIA']['constant_weather']:
            weather_yr_scenario = config['CONFIG']['weather_yr_scenario']
        else:
            weather_yr_scenario = sim_yr

        # --------------------------------------
        # Update result_paths and create folders
        # --------------------------------------
        path_folder_weather_yr = os.path.join(path_new_scenario, str("simulation_results"))

        data['weather_yr_result_paths'] = basic_functions.get_result_paths(
            path_folder_weather_yr)

        folders_to_create = [
            data['weather_yr_result_paths']['data_results'],
            data['weather_yr_result_paths']['data_results_validation'],
            data['weather_yr_result_paths']['data_results_model_run_results_txt']]

        for folder in folders_to_create:
            basic_functions.create_folder(folder)

        technologies = general_assumptions.update_technology_assumption(
            data['assumptions'].technologies,
            narrative_f_eff_achieved=data['assumptions'].non_regional_vars['f_eff_achieved'][sim_yr],
            narrative_gshp_fraction=data['assumptions'].non_regional_vars['gshp_fraction'][sim_yr],
            crit_narrative_input=False)
        data['assumptions'].technologies.update(technologies)

        # ------------------------------------------
        # Run model
        # -------------------------------------------
        sim_obj = energy_demand_model(
            region_selection,
            data,
            config['CRITERIA'],
            data['assumptions'],
            data['weather_stations'],
            weather_yr=weather_yr_scenario,
            weather_by=data['assumptions'].weather_by)

        # ------------------------------------------------
        # Temporal Validation
        # ------------------------------------------------
        if (config['CRITERIA']['validation_criteria'] == True and sim_yr == data['assumptions'].base_yr) and config['CRITERIA']['cluster_calc'] != True:
            lad_validation.spatio_temporal_val(
                sim_obj.ed_fueltype_national_yh,
                sim_obj.ed_fueltype_regs_yh,
                data['weather_yr_result_paths'],
                data['paths'],
                region_selection,
                data['assumptions'].seasons,
                data['assumptions'].model_yeardays_daytype,
                config['CRITERIA']['plot_crit'])

        # -------------------------------------
        # # Generate YAML file with keynames for `sector_model`
        # -------------------------------------
        if config['CRITERIA']['mode_constrained']:
            supply_results = demand_supply_interaction.constrained_results(
                sim_obj.results_constrained,
                sim_obj.results_unconstrained,
                data['assumptions'].submodels_names,
                data['assumptions'].technologies)
        else:
            supply_results = demand_supply_interaction.unconstrained_results(
                sim_obj.results_unconstrained,
                data['assumptions'].submodels_names)

        # --------------------------
        # Write out all calculations
        # --------------------------
        if config['CRITERIA']['write_txt_additional_results']:
            if config['CRITERIA']['crit_plot_enduse_lp']:

                # Maybe move to result folder in a later step
                path_folder_lp = os.path.join(data['weather_yr_result_paths']['data_results'], 'individual_enduse_lp')
                basic_functions.delete_folder(path_folder_lp)
                basic_functions.create_folder(path_folder_lp)
                winter_week, _, _, _ = date_prop.get_seasonal_weeks()

                # Plot electricity
                fueltype_int_elec = lookup_tables.basic_lookups()['fueltypes']['electricity']
                for enduse, ed_yh in sim_obj.tot_fuel_y_enduse_specific_yh.items():
                    fig_enduse_yh.run(
                        name_fig="individ__electricity_{}_{}".format(enduse, sim_yr),
                        path_result=path_folder_lp,
                        ed_yh=ed_yh[fueltype_int_elec],
                        days_to_plot=winter_week)

            # -------------------------------------------
            # Write annual results to txt files
            # -------------------------------------------
            print("... Start writing results to file: " + str(data['weather_yr_result_paths']['data_results_model_run_results_txt']))
            plot_only_selection = True 
            if plot_only_selection:

                # Write only total region
                write_data.write_only_peak_total_regional(
                    sim_yr,
                    "only_total",
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.ed_fueltype_regs_yh,
                    'tot_fueltype_reg')

                # Write only peak day
                write_data.write_only_peak(
                    sim_yr,
                    "only_peak",
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.ed_fueltype_regs_yh,
                    'fueltype_reg_peak_day')

                # Write every hour in year for all regions and fueltypes (fueltype, reg, 8760)
                write_data.write_fueltype_reg_8760(
                    sim_yr,
                    "only_fueltype_reg_8760",
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.ed_fueltype_regs_yh,
                    'fueltype_reg_8760')

                # Write only peak day for space and water heating
                write_data.write_space_and_water_heating(
                    sim_yr,
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.tot_fuel_y_enduse_specific_yh,
                    "out_enduse_specific")
                
                # Check peak
                fueltype_int = tech_related.get_fueltype_int('electricity')
                national_hourly_demand = np.sum(sim_obj.ed_fueltype_regs_yh[fueltype_int], axis=0)
                peak_day_electricity, _ = enduse_func.get_peak_day_single_fueltype(national_hourly_demand)
                selected_hours = date_prop.convert_yearday_to_8760h_selection(peak_day_electricity)
            else:
                #write_data.write_residential_tot_demands(
                #    sim_yr,
                #    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                #    sim_obj.ed_residential_tot_reg_y,
                #    "ed_residential_tot_reg_y")
                write_data.write_supply_results(
                    sim_yr,
                    "ed_fueltype_regs_yh",
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.ed_fueltype_regs_yh,
                    "result_tot_submodels_fueltypes")
                write_data.write_enduse_specific(
                    sim_yr,
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    sim_obj.tot_fuel_y_enduse_specific_yh,
                    "out_enduse_specific")
                write_data.write_lf(
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    "result_reg_load_factor_y",
                    [sim_yr],
                    sim_obj.reg_load_factor_y,
                    'reg_load_factor_y')
                write_data.write_lf(
                    data['weather_yr_result_paths']['data_results_model_run_results_txt'],
                    "result_reg_load_factor_yd",
                    [sim_yr],
                    sim_obj.reg_load_factor_yd,
                    'reg_load_factor_yd')

            print("... Finished writing results to file")

    print("-------------------------")
    print("... Finished running HIRE")
    print("-------------------------")



"""
"""
import logging
import matplotlib.pyplot as plt
import numpy as np

from energy_demand.plotting import basic_plot_functions

def order_polygon(upper_boundary, lower_boundary):
    """Create correct sorting to draw filled polygon

    Arguments
    ---------
    upper_boundary
    lower_boundary

    Returns
    -------
    """
    min_max_polygon = []
    for pnt in upper_boundary:
        min_max_polygon.append(pnt)
    for pnt in reversed(lower_boundary):
        min_max_polygon.append(pnt)

    return min_max_polygon

def create_min_max_polygon_from_lines(line_data):
    """

    Arguments
    ---------
    line_data : dict
        linedata containing info
            {'x_value': [y_values]}

    """
    upper_boundary = []
    lower_bdoundary = []

    for x_value, y_value in line_data.items():
        min_y = np.min(y_value)
        max_y = np.max(y_value)
        upper_boundary.append((x_value, min_y))
        lower_bdoundary.append((x_value, max_y))

        # create correct sorting to draw filled polygon
        min_max_polygon = order_polygon(upper_boundary, lower_bdoundary)

    return min_max_polygon

def plot_lf_y(
        fueltype_int,
        fueltype_str,
        reg_load_factor_y,
        reg_nrs,
        path_plot_fig,
        plot_individ_lines=False,
        plot_max_min_polygon=True
    ):
    """Plot load factors per region for every year

    Arguments
    --------

    """
    print("... plotting load factors")

    # Set figure size
    fig = plt.figure(figsize=basic_plot_functions.cm2inch(8, 8))

    ax = fig.add_subplot(1, 1, 1)

    if plot_individ_lines:
        # Line plot for every region over years
        for reg_nr in range(reg_nrs):
            x_values_year = []
            y_values_year = []

            for year, lf_fueltype_reg in reg_load_factor_y.items():
                x_values_year.append(year)
                y_values_year.append(lf_fueltype_reg[fueltype_int][reg_nr])

            plt.plot(
                x_values_year,
                y_values_year,
                linewidth=0.2,
                color='grey')

    if plot_max_min_polygon:
        '''lower_bdoundary = []
        upper_boundary = []

        for year_nr, lf_fueltype_reg in reg_load_factor_y.items():

            # Get min and max of all entries of year of all regions
            min_y = np.min(lf_fueltype_reg[fueltype_int])
            max_y = np.max(lf_fueltype_reg[fueltype_int])
            upper_boundary.append((year_nr, min_y))
            lower_bdoundary.append((year_nr, max_y))

        # create correct sorting to draw filled polygon
        min_max_polygon = order_polygon(upper_boundary, lower_bdoundary)'''

        min_max_polygon = create_min_max_polygon_from_lines(reg_load_factor_y)

        polygon = plt.Polygon(
            min_max_polygon,
            color='grey',
            alpha=0.2,
            edgecolor=None,
            linewidth=0,
            fill='True')

        ax.add_patch(polygon)
    # -----------------
    # Axis
    # -----------------
    plt.ylim(0, 100)

    # -----------------
    # Axis labelling
    # -----------------
    plt.xlabel("years")
    plt.ylabel("load factor, fueltpye {} [%]".format(fueltype_str))

    years = list(reg_load_factor_y.keys())
    base_yr = 2015

    # Major ticks
    major_interval = 10
    major_ticks = np.arange(base_yr, years[-1] + major_interval, major_interval)
    ax.set_xticks(major_ticks)

    # Minor ticks
    minor_interval = 5
    minor_ticks = np.arange(base_yr, years[-1] + minor_interval, minor_interval)
    ax.set_xticks(minor_ticks, minor=True)

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    plt.savefig(path_plot_fig)
    plt.close()

def plot_seasonal_lf(
        fueltype_int,
        fueltype_str,
        load_factors_seasonal,
        reg_nrs,
        path_plot_fig,
        plot_individ_lines=False,
        plot_max_min_polygon=True
    ):
    """Plot load factors per region for every year

    Arguments
    --------
    fueltype_int : int
        Fueltype_int to print (see lookup)
    fueltype_str : str
        Fueltype string to print
    load_factors_seasonal : dict
        Seasonal load factors per season
    reg_nrs : int
        Number of region
    """
    print("... plotting seasonal load factors")

    # Set figure size
    fig = plt.figure(figsize=basic_plot_functions.cm2inch(8, 8))
    ax = fig.add_subplot(1, 1, 1)

    # Settings
    color_list = {
        'winter': 'midnightblue',
        'summer': 'olive',
        'spring': 'darkgreen',
        'autumn': 'gold'}

    classes = list(color_list.keys())
    #class_colours = list(color_list.values())

    # ------------
    # Iterate regions and plot load factors for every region
    # ------------
    if plot_individ_lines:
        for reg_nr in range(reg_nrs):
            for season, lf_fueltypes_season in load_factors_seasonal.items():

                x_values_season_year = []
                y_values_season_year = []
                for year, lf_fueltype_reg in lf_fueltypes_season.items():
                    x_values_season_year.append(year)
                    y_values_season_year.append(lf_fueltype_reg[fueltype_int][reg_nr])

                # plot individual saisonal data point
                plt.plot(
                    x_values_season_year,
                    y_values_season_year,
                    color=color_list[season],
                    linewidth=0.2,
                    alpha=0.2)

    # -----------------
    # Plot min_max_area
    # -----------------
    if plot_max_min_polygon:

        for season, lf_fueltypes_season in load_factors_seasonal.items():
            upper_boundary = []
            lower_bdoundary = []

            min_max_polygon = create_min_max_polygon_from_lines(lf_fueltypes_season)

            '''for year_nr, lf_fueltype_reg in lf_fueltypes_season.items():

                # Get min and max of all entries of year of all regions
                min_y = np.min(lf_fueltype_reg[fueltype_int])
                max_y = np.max(lf_fueltype_reg[fueltype_int])
                upper_boundary.append((year_nr, min_y))
                lower_bdoundary.append((year_nr, max_y))

            # create correct sorting to draw filled polygon
            min_max_polygon = order_polygon(upper_boundary, lower_bdoundary)'''

            polygon = plt.Polygon(
                min_max_polygon,
                color=color_list[season],
                alpha=0.2,
                edgecolor=None,
                linewidth=0,
                fill='True')

            ax.add_patch(polygon)

    # ------------------------------------
    # Calculate average per season for all regions
    # and plot average line a bit thicker
    # ------------------------------------
    for season in classes:
        years = []
        average_season_year_years = []
        for year in load_factors_seasonal[season].keys():
            average_season_year = []

            # Iterate over regions
            for reg_nr in range(reg_nrs):
                average_season_year.append(
                    load_factors_seasonal[season][year][fueltype_int][reg_nr])

            years.append(int(year))
            average_season_year_years.append(np.average(average_season_year))

        # plot average
        plt.plot(
            years,
            average_season_year_years,
            color=color_list[season],
            linewidth=0.5,
            linestyle='--',
            alpha=1.0,
            markersize=0.5,
            marker='o',
            label=season)

        # Plot markers for average line
        '''plt.plot(
            years,
            average_season_year_years,
            color=color_list[season],
            markersize=0.5,
            linewidth=0.5,
            marker='o')'''

    # -----------------
    # Axis
    # -----------------
    plt.ylim(0, 100)

    # -----------------
    # Axis labelling and ticks
    # -----------------
    plt.xlabel("years")
    plt.ylabel("load factor {} [%]".format(fueltype_str))

    base_yr = 2015
    minor_interval = 5
    major_interval = 10

    # Major ticks
    major_ticks = np.arange(base_yr,years[-1] + major_interval, major_interval)
    ax.set_xticks(major_ticks)
    #ax.set_xlabel(major_ticks)

    # Minor ticks
    minor_ticks = np.arange(base_yr,years[-1] + minor_interval, minor_interval)
    ax.set_xticks(minor_ticks, minor=True)
    #ax.set_xlabel(minor_ticks)

    # ------------
    # Plot color legend with colors for every season
    # ------------
    plt.legend(
        ncol=2,
        prop={
            'family': 'arial',
            'size': 5},
        loc='best',
        frameon=False)

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    # Save fig
    plt.savefig(path_plot_fig)
    plt.close()


"""Validation plots
"""
# print dataframe
import matplotlib.pyplot as plt

def plot_dataframe_function(
        my_df,
        x_column_name,
        y_column_names,
        from_to_value_to_plot=[],
        plot_kind='line'):
    """
    """
    if from_to_value_to_plot == []:

        result_to_plot = my_df.plot(
            x=x_column_name,
            y=y_column_names,
            kind=plot_kind)
    else:
        from_value = from_to_value_to_plot[0]
        to_value = from_to_value_to_plot[1]

        result_to_plot = my_df[from_value:to_value].plot(
            x=x_column_name,
            y=y_column_names,
            kind=plot_kind)

    # ------------------------------------- 
    # Legend
    # -------------------------------------
    if type(y_column_names) is list:
        legend_entries = y_column_names
    else:
        legend_entries = [y_column_names]

    plt.legend(
        legend_entries,
        ncol=1,
        frameon=False,
        prop={
            'family': 'arial',
            'size': 15})
    
    plt.axis('tight')

    #plt.show()

    print("finished plotting")

    #plt.ylabel("GW")
    #plt.xlabel("day")
    #plt.title("tot annual ED, all enduses, fueltype {}".format(year_to_plot + 2050))
    #plt.savefig(fig_name)
    #plt.close()

'''
# Sum across all regions
    sum_across_regions = results_unconstrained.sum(axis=1)

    rows = []

    for hour in range(8760):

        # Get day and hour
        day_year, hour_day_year = date_prop.convert_h_to_day_year_and_h(hour)

        # Start row
        row = {'year': year, 'hour': hour}

        for submodel_nr, submodel in enumerate(submodels):

            # Total energy demand
            ed_submodel_h = sum_across_regions[submodel_nr][fuelype_nr][hour]

            # Space heating related demand for sector
            if submodel_nr == 0:
                space_heating_demand = enduse_specific_results['rs_space_heating'][fuelype_nr][day_year][hour_day_year]
            elif submodel_nr == 1:
                space_heating_demand = enduse_specific_results['ss_space_heating'][fuelype_nr][day_year][hour_day_year]
            else:
                space_heating_demand = enduse_specific_results['is_space_heating'][fuelype_nr][day_year][hour_day_year]

            ed_submodel_heating_h = space_heating_demand
            str_name_heat = "{}_heat".format(submodel)
            row[str_name_heat] = ed_submodel_heating_h

            # Non-heating related demand
            ed_submodel_non_heating_h = ed_submodel_h - space_heating_demand

            str_name_non_heat = "{}_non_heat".format(submodel)

            row[str_name_non_heat] = ed_submodel_non_heating_h

        rows.append(row)

    # Create dataframe
    col_names = [
        'year',
        'hour',
        'residential_non_heat',
        'residential_heat',
        'service_non_heat',
        'service_heat',
        'industry_non_heat',
        'industry_heat']

    my_df = pd.DataFrame(rows, columns=col_names)
    my_df.to_csv(path, index=False) #Index prevents writing index rows

'''


import os
import configparser
import csv
import yaml
import numpy as np
from ruamel.yaml import YAML

from energy_demand.read_write import write_data


def run(
        path,
        sos_model,
        narrative,
        weather_name,
        gva_scenario,
        population_scenario,
        timesteps
    ):

    # Narrative parameters
    narratives = {
        'air_leakage': [narrative],
        'assump_diff_floorarea_pp': [narrative],
        'cooled_floorarea': [narrative],
        'dm_improvement': [narrative],
        'f_eff_achieved': [narrative],
        'generic_enduse_change': [narrative],
        'heat_recovered': [narrative],
        't_base_heating': [narrative],
        'p_cold_rolling_steel': [narrative],
        'smart_meter_p': [narrative],
        'generic_fuel_switch': [narrative]
        }

    scenario_name = 'energy_demand_constrained__{}__{}__{}'.format(weather_name, population_scenario, gva_scenario)

    yaml_dict_out = {
        'name': scenario_name,
        'description': "",
        'sos_model': sos_model,
        'stamp': "",
        'decision_module': '',
        'narratives': narratives,
        'strategies': {},
        'timesteps': timesteps,
        'scenarios': {
            'gva_per_head': gva_scenario,
            'population': population_scenario,
            'temperature': weather_name,
            'weather_station_coordinates': 'weather_station_coordinates__{}'.format(weather_name)
        }
    }


    path_out_file = os.path.join(path, "{}.yml".format(scenario_name))
    write_data.write_yaml(yaml_dict_out, path_out_file)

path = "C:/_scrap"

for i in range(1, 101, 1):

    # Narrative
    narrative = "central_narrative"

    # Scenario data
    gva_scenario = 'gva_baseline'
    population_scenario = 'pop_low'
    timesteps = [2015, 2050]
    sos_model = "energy_demand_constrained"
    weather_realisation = "NF{}".format(i)

    # Create .yml file
    run(
        path,
        sos_model,
        narrative,
        weather_realisation,
        gva_scenario,
        population_scenario,
        timesteps)


"""
Contains funtions related to handeling date and
time related functionality
"""
from datetime import date
from datetime import timedelta
import numpy as np

from energy_demand.basic import basic_functions

def convert_yearday_to_8760h_selection(yearday):
    """Select from 8760 hours the 24 hours of yearday

    Arguments
    ---------
    yearday : int
        Range between 0 and 364
    """
    hour_start = yearday * 24
    hour_end = (yearday + 1) * 24

    hours = range(hour_start, hour_end, 1)
    return hours

def convert_h_to_day_year_and_h(hour):
    """Convert a 8760 hour in a year
    to the yearday and hour of that day

    Arguments
    -----
    hour : int
        Hour of 8760 hours in a year

    Outputs
    -------
    day : int
        Pthon day of a year (0 == 1 day)
    day_h : int
        Python hour of day (0 == 1 hour)
    """
    day = basic_functions.round_down(int(hour / 24), 1)
    total_hours = day * 24
    day_h = hour - total_hours

    return day, day_h

def get_8760_hrs_from_yeardays(yeardays):
    """Get from yeadays all yearhours

    Arguments
    ---------
    yeardays : list
        Listh with year days (0 - 364)

    Returns
    ---------
    year_hrs : list
        All year hours (0 - 8759)
    """
    year_hrs = []
    for day in yeardays:
        hrs = list(range((day * 24), ((day + 1) * 24)))
        year_hrs += hrs

    return year_hrs

def get_seasonal_weeks():
    """
    """
    '''winter_week = list(range(
        date_prop.date_to_yearday(year_to_model, 1, 12),
        date_prop.date_to_yearday(year_to_model, 1, 26))) #Jan
    spring_week = list(range(
        date_prop.date_to_yearday(year_to_model, 5, 11),
        date_prop.date_to_yearday(year_to_model, 5, 25))) #May
    summer_week = list(range(
        date_prop.date_to_yearday(year_to_model, 7, 13),
        date_prop.date_to_yearday(year_to_model, 7, 27))) #Jul
    autumn_week = list(range(
        date_prop.date_to_yearday(year_to_model, 10, 12),
        date_prop.date_to_yearday(year_to_model, 10, 26))) #Oct'''
    winter_week = list(range(
        date_to_yearday(2015, 1, 12), date_to_yearday(2015, 1, 19))) #Jan
    spring_week = list(range(
        date_to_yearday(2015, 5, 11), date_to_yearday(2015, 5, 18))) #May
    summer_week = list(range(
        date_to_yearday(2015, 7, 13), date_to_yearday(2015, 7, 20))) #Jul
    autumn_week = list(range(
        date_to_yearday(2015, 10, 12), date_to_yearday(2015, 10, 19))) #Oct

    return winter_week, spring_week, summer_week, autumn_week

def get_yeardays_daytype(year_to_model):
    """For yearday, daytype and date related
    properties

    Arguments
    ---------
    year_to_model : int
        Year to get daytpes

    Returns
    -------
    model_yeardays_daytype : dict
        Dict with description for every day if working or holiday
    yeardays_month : dict
        For every month the yeardays (as int) are provided
    yeardays_month_days : dict
        For every month, all dates are provided as date objects
    
    Note
    ----
    Only works with years with 365 days so far
    """
    # --------------------------------------
    # Calculate for all yeardays the daytype of base year
    # --------------------------------------
    list_dates = fullyear_dates(
        start=date(year_to_model, 1, 1),
        end=date(year_to_model, 12, 31))

    model_yeardays_daytype = np.array(['workday']*365)

    # Take respectve daily fuel curve depending on weekday or weekend
    for array_day, date_yearday in enumerate(list_dates):
        daytype = get_weekday_type(date_yearday)
        if daytype == 'holiday':
            model_yeardays_daytype[array_day] = 'holiday'

    # Calculate month of dates
    yeardays_month_days = {}
    for i in range(12):
        yeardays_month_days[i] = []

    # Get month type of yearday for every day
    yeardays_month = []
    for yearday, date_object in enumerate(list_dates):
        month_yearday = date_object.timetuple().tm_mon - 1
        yeardays_month.append(month_yearday)
        yeardays_month_days[month_yearday].append(yearday)

    return model_yeardays_daytype, yeardays_month, yeardays_month_days

def get_season(year_to_model):
    """

    Arguments
    ---------
    year_to_model : int
        Year which is modelled
    """
    # Full meteorological seasons
    seasons = {}
    seasons['winter'] = list(
        range(
            date_to_yearday(year_to_model, 12, 1),
            date_to_yearday(year_to_model, 12, 31))) + list(
                range(
                    date_to_yearday(year_to_model, 1, 1),
                    date_to_yearday(year_to_model, 2, 28)))
    seasons['spring'] = list(range(
        date_to_yearday(year_to_model, 3, 1),
        date_to_yearday(year_to_model, 5, 31)))
    seasons['summer'] = list(range(
        date_to_yearday(year_to_model, 6, 1),
        date_to_yearday(year_to_model, 8, 31)))
    seasons['autumn'] = list(range(
        date_to_yearday(year_to_model, 9, 1),
        date_to_yearday(year_to_model, 11, 30)))

    return seasons

def get_month_from_yeraday(year, yearday):
    """Get python month from yearday

    Arguments
    ----------
    year : int
        year
    yearday : int
        yearday

    Return
    ------
    python_month : int
        Python month (0 == 1)
    """
    date_year = yearday_to_date(int(year), int(yearday))
    python_month = date_year.timetuple().tm_mon - 1

    return python_month

def is_leap_year(year):
    """Determine whether a year is a leap year"""
    return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)

def date_to_yearday(year, month, day):
    """Gets the yearday (julian year day) of a year minus one to correct because of python iteration

    Arguments
    ----------
    date_base_yr : int
        Year
    date_base_yr : int
        Month
    day : int
        Day

    Example
    -------
    5. January 2015 --> Day nr 5 in year --> -1 because of python --> Out: 4
    """
    date_y = date(year, month, day)
    yearday = date_y.timetuple().tm_yday - 1 #: correct because of python iterations

    return yearday

def yearday_to_date(year, yearday_python):
    """Gets the yearday of a year minus one to correct
    because of python iteration

    Arguments
    ----------
    year : int
        Year
    yearday_python : int
        Yearday - 1

    Returns
    -------
    date_new : date
        Date
    """
    date_new = date(year, 1, 1) + timedelta(yearday_python)

    return date_new

def get_weekday_type(date_to_test):
    """Gets the weekday of a date

    Arguments
    ----------
    date_to_test : date
        Date of a day in ayear

    Returns
    -------
    daytype : str
        holiday or working day

    Note
    ----
    Bank holidays are defined for the year 2002 - 2015. The whole week of christmas
    is defined as a bank holiday

    Info
    -----
    timetuple attributes:
        tm_year
        tm_mon
        tm_mday
        tm_hour
        tm_min
        tm_sec
        tm_wday
        tm_yday
        tm_isdst
    """
    weekday = date_to_test.timetuple().tm_wday

    if weekday == 5 or weekday == 6:
        return 'holiday'
    else:
        year = date_to_test.timetuple().tm_year
        if year == 2015:
            bank_holidays = [
                date(2015, 1, 1),
                date(2015, 1, 2),
                date(2015, 4, 3),
                date(2015, 4, 6),
                date(2015, 5, 4),
                date(2015, 5, 25),
                date(2015, 8, 31),
                date(2015, 12, 24),
                date(2015, 12, 25),
                date(2015, 12, 26),
                date(2015, 12, 27),
                date(2015, 12, 28),
                date(2015, 12, 29),
                date(2015, 12, 30),
                date(2015, 12, 31)]
        elif year == 2014:
            bank_holidays = [
                date(2014, 1, 1),
                date(2014, 1, 2),
                date(2014, 4, 18),
                date(2014, 4, 21),
                date(2014, 5, 5),
                date(2014, 5, 26),
                date(2014, 8, 25),
                date(2014, 12, 24),
                date(2014, 12, 25),
                date(2014, 12, 26),
                date(2014, 12, 27),
                date(2014, 12, 28),
                date(2014, 12, 29),
                date(2014, 12, 30),
                date(2014, 12, 31)]
        elif year == 2013:
            bank_holidays = [
                date(2013, 1, 1),
                date(2013, 1, 2),
                date(2013, 4, 29),
                date(2013, 4, 1),
                date(2013, 5, 6),
                date(2013, 5, 27),
                date(2013, 8, 26),
                date(2013, 12, 24),
                date(2013, 12, 25),
                date(2013, 12, 26),
                date(2013, 12, 27),
                date(2013, 12, 28),
                date(2013, 12, 29),
                date(2013, 12, 30),
                date(2013, 12, 31)]
        elif year == 2012:
            bank_holidays = [
                date(2012, 1, 1),
                date(2012, 1, 2),
                date(2012, 4, 6),
                date(2012, 4, 9),
                date(2012, 5, 7),
                date(2012, 6, 4),
                date(2012, 6, 7),
                date(2012, 8, 27),
                date(2012, 12, 24),
                date(2012, 12, 25),
                date(2012, 12, 26),
                date(2012, 12, 27),
                date(2012, 12, 28),
                date(2012, 12, 29),
                date(2012, 12, 30),
                date(2012, 12, 31)]
        elif year == 2011:
            bank_holidays = [
                date(2011, 1, 1),
                date(2011, 1, 2),
                date(2011, 1, 3),
                date(2011, 4, 22),
                date(2011, 4, 25),
                date(2011, 4, 29),
                date(2011, 5, 2),
                date(2011, 5, 30),
                date(2011, 8, 29),
                date(2011, 12, 24),
                date(2011, 12, 25),
                date(2011, 12, 26),
                date(2011, 12, 27),
                date(2011, 12, 28),
                date(2011, 12, 29),
                date(2011, 12, 30),
                date(2011, 12, 31)]
        elif year == 2010:
            bank_holidays = [
                date(2010, 1, 1),
                date(2010, 4, 2),
                date(2010, 4, 5),
                date(2010, 5, 3),
                date(2010, 5, 31),
                date(2010, 8, 30),
                date(2010, 12, 24),
                date(2010, 12, 25),
                date(2010, 12, 26),
                date(2010, 12, 27),
                date(2010, 12, 28),
                date(2010, 12, 29),
                date(2010, 12, 30),
                date(2010, 12, 31)]
        elif year == 2009:
            bank_holidays = [
                date(2009, 1, 1),
                date(2009, 4, 10),
                date(2009, 4, 13),
                date(2009, 5, 4),
                date(2009, 5, 25),
                date(2009, 8, 31),
                date(2009, 12, 24),
                date(2009, 12, 25),
                date(2009, 12, 26),
                date(2009, 12, 27),
                date(2009, 12, 28),
                date(2009, 12, 29),
                date(2009, 12, 30),
                date(2009, 12, 31)]
        elif year == 2008:
            bank_holidays = [
                date(2008, 1, 1),
                date(2008, 4, 21),
                date(2008, 4, 24),
                date(2008, 5, 5),
                date(2008, 5, 26),
                date(2008, 8, 25),
                date(2008, 12, 24),
                date(2008, 12, 25),
                date(2008, 12, 26),
                date(2008, 12, 27),
                date(2008, 12, 28),
                date(2008, 12, 29),
                date(2008, 12, 30),
                date(2008, 12, 31)]
        elif year == 2007:
            bank_holidays = [
                date(2007, 1, 1),
                date(2007, 4, 6),
                date(2007, 4, 9),
                date(2007, 5, 7),
                date(2007, 5, 28),
                date(2007, 8, 27),
                date(2007, 12, 24),
                date(2007, 12, 25),
                date(2007, 12, 26),
                date(2007, 12, 27),
                date(2007, 12, 28),
                date(2007, 12, 29),
                date(2007, 12, 30),
                date(2007, 12, 31)]
        elif year == 2006:
            bank_holidays = [
                date(2006, 1, 2),
                date(2006, 4, 14),
                date(2006, 4, 17),
                date(2006, 5, 1),
                date(2006, 5, 29),
                date(2006, 8, 28),
                date(2006, 12, 24),
                date(2006, 12, 25),
                date(2006, 12, 26),
                date(2006, 12, 27),
                date(2006, 12, 28),
                date(2006, 12, 29),
                date(2006, 12, 30),
                date(2006, 12, 30)]
        elif year == 2005:
            bank_holidays = [
                date(2005, 1, 1),
                date(2005, 1, 2),
                date(2005, 1, 3),
                date(2005, 3, 25),
                date(2005, 3, 28),
                date(2005, 5, 2),
                date(2005, 5, 30),
                date(2005, 8, 29),
                date(2005, 12, 24),
                date(2005, 12, 25),
                date(2005, 12, 26),
                date(2005, 12, 27),
                date(2005, 12, 28),
                date(2005, 12, 29),
                date(2005, 12, 30),
                date(2005, 12, 31)]
        elif year == 2004:
            bank_holidays = [
                date(2004, 1, 1),
                date(2004, 4, 9),
                date(2004, 4, 12),
                date(2004, 4, 25),
                date(2004, 5, 3),
                date(2004, 5, 31),
                date(2004, 8, 30),
                date(2004, 12, 24),
                date(2004, 12, 25),
                date(2004, 12, 26),
                date(2004, 12, 27),
                date(2004, 12, 28),
                date(2004, 12, 29),
                date(2004, 12, 30),
                date(2004, 12, 31)]
        elif year == 2003:
            bank_holidays = [
                date(2003, 1, 1),
                date(2003, 4, 21),
                date(2003, 5, 5),
                date(2003, 5, 26),
                date(2003, 8, 25),
                date(2003, 12, 24),
                date(2003, 12, 25),
                date(2003, 12, 26),
                date(2003, 12, 27),
                date(2003, 12, 28),
                date(2003, 12, 29),
                date(2003, 12, 30),
                date(2003, 12, 31)]
        elif year == 2002:
            bank_holidays = [
                date(2002, 1, 1),
                date(2002, 4, 1),
                date(2002, 5, 6),
                date(2002, 6, 3),
                date(2002, 6, 4),
                date(2002, 8, 26),
                date(2002, 12, 24),
                date(2002, 12, 25),
                date(2002, 12, 26),
                date(2002, 12, 27),
                date(2002, 12, 28),
                date(2002, 12, 29),
                date(2002, 12, 30),
                date(2002, 12, 31)]
        else:
            bank_holidays = []

        if date_to_test in bank_holidays:
            return 'holiday'
        else:
            return 'working_day'

def fullyear_dates(start, end):
    """Calculates all dates between a star and end date.
    The star and end date are included in the list.

    Arguments
    ----------
    start : date
        Start date
    end : date
        End date

    Returns
    -------
    list_dates : list
        A list with all daily dates
    """
    list_dates = []
    span = end - start
    for day in range(span.days + 1):
        list_dates.append(start + timedelta(days=day))

    return list_dates
 


"""This file creates a .zip folder with all the necessary data files to run HIRE
for a specific version
"""
import os
import sys
import zipfile

from energy_demand.basic import basic_functions

def zipdir(path, zip_handler):
    """Zip a whole directory
    """
    for root, dirs, files in os.walk(path):
        for file in files:
            zip_handler.write(
                filename=os.path.join(root, file),
                arcname=os.path.relpath(
                    os.path.join(root, file),
                    os.path.join(path, '..')))

    return zip_handler

def zipit(dir_list, zip_name):
    """Zip a list with directories
    """
    zip_handler = zipfile.ZipFile(zip_name, 'w', zipfile.ZIP_DEFLATED)
    for dir_to_zip in dir_list:
        print("... zipping folder: {}".format(dir))
        zip_handler = zipdir(dir_to_zip, zip_handler)

    # Close zip
    zip_handler.close()

def package_data(
        version_name,
        data_folder_path#="C:/Users/cenv0553/ed/data"
    ):
    """
    verions_name : str
        Name of version
    data_folder_path : str
        Path to store packaged data
    """
    print("... start packaging data version: {}".format(version_name))

    # Delete all processes files
    path_folder = os.path.join(data_folder_path, "scenarios", "MISTRAL_pop_gva")
    basic_functions.delete_folder(path_folder)

    # Names
    zip_name_full = os.path.join(data_folder_path, "{}_{}".format(version_name, "full.zip"))
    zip_name_minimum = os.path.join(data_folder_path, "{}_{}".format(version_name, "minimum.zip"))

    # Zip minimum files
    _raw_folders_data_minimal = [
        #'_old_data',
        #'_raw_data',
        #'00_user_defined_variables',
        #'00_user_defined_variables_P3',
        #'coefficients',
        #'config_data',
        #'dimensions',
        #'initial_conditions',
        #'initial_inputs',
        #'interval_definitions',
        #'interventions',
        'energy_demand_minimal',
        #'narratives',
        #'parameters',
        #'region_definitions',
        #'scenarios',
        #'strategies'
        ]
    

    # Zip maximum files
    _raw_folders_data_full = [
        '_old_data',
        '_raw_data',
        '00_user_defined_variables',
        '00_user_defined_variables_P3',
        'coefficients',
        'config_data',
        'dimensions',
        'initial_conditions',
        'initial_inputs',
        'interval_definitions',
        'interventions',
        'energy_demand_minimal',
        'narratives',
        'parameters',
        'region_definitions',
        'scenarios',
        'strategies'
        ]

    paths_minimal = []
    for folder in _raw_folders_data_minimal:
        path_folder = os.path.join(data_folder_path, folder)
        paths_minimal.append(path_folder)

    paths_full = []
    for folder in _raw_folders_data_full:
        path_folder = os.path.join(data_folder_path, folder)
        paths_full.append(path_folder)

    # Zip minimal
    zipit(
        dir_list=paths_minimal,
        zip_name=zip_name_minimum)

    # -------------------------------------------
    # Add folders and rename it
    # -------------------------------------------
    folders_to_add = (
        ('energy_demand_minimal', 'energy_demand'),
        ('scenarios_minimal', 'scenarios'))

    zip_handler_minimum = zipfile.ZipFile(os.path.join(data_folder_path, zip_name_minimum), "a")

    for folder_name_to_add, renamed_folder in folders_to_add:
        folder_to_add = os.path.join(data_folder_path, folder_name_to_add)

        for root, dirs, files in os.walk(folder_to_add):
            for file in files:

                # New path
                root_renamed = root.replace(folder_name_to_add, renamed_folder)

                # Get rid of local paths
                root_renamed_without_local_path = root_renamed.split(data_folder_path)[1]
                root_renamed_without_local_path = root_renamed_without_local_path.split('energy_demand')[1]
                new_path = os.path.join(root_renamed_without_local_path, file)

                zip_handler_minimum.write(
                    filename=os.path.join(root, file),
                    arcname=new_path)

    # Close zip
    #zip_handler_minimum.close()

    # Zip full
    zipit(
        dir_list=paths_full,
        zip_name=zip_name_full)

    # ------------
    # Append individual files
    # ------------
    #zip_handler_minimum = zipfile.ZipFile(os.path.join(data_folder_path, zip_name_minimum), "a")
    zip_handler_full = zipfile.ZipFile(os.path.join(data_folder_path, zip_name_full), "a")

    # Add units file
    full_file_path = os.path.join(data_folder_path, 'units.txt')
    zip_handler_full.write(full_file_path, arcname='units.txt')
    zip_handler_minimum.write(full_file_path, arcname='units.txt')

    # Add other files to full data
    files_to_add_full = [
        'population-economic-smif-csv-from-nismod-db.zip']

    for file_to_add in files_to_add_full:
        full_file_path = os.path.join(data_folder_path, file_to_add)
        zip_handler_full.write(full_file_path, arcname=file_to_add)

    zip_handler_full.close()
    zip_handler_minimum.close()

    print("Finished packaging data for Version {}".format(version_name))

if __name__ == '__main__':
    """Provide version name and path to data folder

    E.g.:

    python ../generate_data_version.py v_6_1 C:/path_to_data
    """
    # Map command line arguments to function arguments.
    #package_data(*sys.argv[1:])
    package_data('v0.8.9', 'C:/Users/cenv0553/ED/data')



"""
"""
import numpy as np
import matplotlib.pyplot as plt

from energy_demand.plotting import basic_plot_functions

def run(
        results_resid,
        lookups,
        hours_to_plot,
        year_to_plot,
        fig_name
    ):
    """Plots stacked end_use for all regions. As
    input GWh per h are provided, which cancels out to
    GW.

    Arguments
    ---------
    year_to_plot : int
        2015 --> 0

    # INFO Cannot plot a single year?
    """
    print("... plot a full week")

    fig, ax = plt.subplots()

    days_to_plot = range(365)

    legend_entries = []

    numer_h = len(hours_to_plot)
    x_values = np.array(range(numer_h))

    y_init = np.zeros((lookups['fueltypes_nr'], numer_h))

    for fueltype_str, fueltype_int in lookups['fueltypes'].items():
        legend_entries.append(fueltype_str)

        # Select year to plot
        fuel_all_regions = results_resid[year_to_plot][fueltype_int]

        data_over_day = np.zeros((8760))
        for region_data in fuel_all_regions:
            data_over_day += region_data

        y_init[fueltype_int] = data_over_day[hours_to_plot] #Select hours

    for line, _ in enumerate(y_init):

        smooth_x_line_data, smooth_y_line_data = basic_plot_functions.smooth_line(
            x_values, y_init[line], nr_line_points=300)

        #plt.plot(y_init[line])
        plt.plot(smooth_x_line_data, smooth_y_line_data)

    ax.legend(legend_entries)

    x_tick_pos = []
    for day in range(365):
        x_tick_pos.append(day * 24)
    plt.xticks(x_tick_pos, days_to_plot, color='black')

    plt.axis('tight')

    plt.legend(
        ncol=2,
        frameon=False,
        prop={
            'family': 'arial',
            'size': 10},)

    plt.ylabel("GW")
    plt.xlabel("day")
    plt.title("tot annual ED, all enduses, fueltype {}".format(year_to_plot + 2050))

    plt.savefig(fig_name)
    plt.close()




import os

def get_size(start_path = '.'):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return total_size

paths = [
    "//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final/h_min",
    "//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final/h_max",
    "//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final/l_min",
    "//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final/l_max",
]
#paths = ["//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final/UNFINISHED"]
for path in paths:
    folders = os.listdir(path)

    for folder in folders:
        path_folder = os.path.join(path, folder)
        folder_size = get_size(path_folder) / 8589934592 #bit to Gigabyte
        print("Size: {}  {}".format(round(folder_size, 3), path_folder), flush=True)


"""Short diverse helper functions
"""
from collections import defaultdict
from energy_demand.basic import basic_functions

def copy_fractions_all_sectors(
        fuel_tech_p_by,
        sectors,
        affected_enduses
    ):
    """Copy all defined fractions
    for an enduse to all setors

    Arguments
    ---------
    fuel_tech_p_by : dict
        Fuel shares per technolgy for an enduse
    sectors : list
        All sectors with this induse where the identical
        shares want to be transferred
    affected_enduses : list
        Enduses for which the values should be copied

    Returns
    -------
    out_dict : dict
        Fuel shares for all sectors
        i.e. {enduse: {sector: {fueltype: {tech: {share}}}}}
    """
    out_dict = defaultdict(dict)
    for sector in sectors:
        for enduse, techs in fuel_tech_p_by.items():
            if enduse in affected_enduses:
                out_dict[enduse][sector] = dict(techs) #dict necessary
            else:
                out_dict[enduse] = dict(techs)
    out_dict = dict(out_dict)

    return out_dict

def init_fuel_tech_p_by(all_enduses_with_fuels, fueltypes_nr):
    """Helper function to define stocks for all enduse and fueltype

    Arguments
    ----------
    all_enduses_with_fuels : dict
        Provided fuels
    fueltypes_nr : int
        Nr of fueltypes

    Returns
    -------
    fuel_tech_p_by : dict

    """
    fuel_tech_p_by = {}

    fueltpe_dicts= dict.fromkeys(range(fueltypes_nr), {})

    for enduse in all_enduses_with_fuels:
        fuel_tech_p_by[enduse] = dict(fueltpe_dicts)

    return fuel_tech_p_by

def init_dict_brackets(first_level_keys):
    """Initialise a  dictionary with one level

    Arguments
    ----------
    first_level_keys : list
        First level data

    Returns
    -------
    one_level_dict : dict
         dictionary
    """
    one_level_dict = {}

    for first_key in first_level_keys:
        one_level_dict[first_key] = {}

    return one_level_dict

def add_undef_techs(heat_pumps, specified_tech_enduse, enduses):
    """Add technology to dict

    Arguments
    ----------
    heat_pumps : list
        List with heat pumps
    specified_tech_enduse_by : dict
        Technologey per enduse
    enduses : list
        Enduses

    Return
    -------
    specified_tech_enduse : dict
        Specified techs per enduse
    """
    for enduse in enduses:
        for heat_pump in heat_pumps:
            for sector in specified_tech_enduse[enduse]:
                if heat_pump not in specified_tech_enduse[enduse][sector]:
                    specified_tech_enduse[enduse][sector].append(heat_pump)

    return specified_tech_enduse

def get_def_techs(fuel_tech_p_by):
    """Collect all technologies across all
    fueltypes for each endues

    Arguments
    ----------
    fuel_tech_p_by : dict
        Fuel share per technology for base year
    sector_crit : bool
        Criteria wheter fuel is given by sector or not

    Returns
    -------
    all_defined_tech_service_ey : dict
        All defined technologies with service in end year
    """
    all_defined_tech_service_ey = {}

    for enduse in fuel_tech_p_by:
        all_defined_tech_service_ey[enduse] = {}

        sector_crit = basic_functions.test_if_sector(
            fuel_tech_p_by[enduse])

        if sector_crit:
            for sector in fuel_tech_p_by[enduse]:
                all_defined_tech_service_ey[enduse][sector] = set()
                for fueltype in fuel_tech_p_by[enduse][sector]:
                    for tech in list(fuel_tech_p_by[enduse][sector][fueltype].keys()):
                        all_defined_tech_service_ey[enduse][sector].add(tech)
                all_defined_tech_service_ey[enduse][sector] = list(all_defined_tech_service_ey[enduse][sector]) 
        else:
            all_defined_tech_service_ey[enduse][None] = set()
            for fueltype in fuel_tech_p_by[enduse]:
                for tech in fuel_tech_p_by[enduse][fueltype]:
                    all_defined_tech_service_ey[enduse][None].add(tech)
            all_defined_tech_service_ey[enduse][None] = list(all_defined_tech_service_ey[enduse][None])

    return all_defined_tech_service_ey

def get_nested_dict_key(nested_dict):
    """Get all keys of nested dict

    Arguments
    ----------
    nested_dict : dict
        Nested dictionary

    Return
    ------
    all_nested_keys : list
        Key of nested dict
    """
    all_nested_keys = []

    for values in nested_dict.values():
        all_nested_keys += values.keys()

    return all_nested_keys

def set_same_eff_all_tech(technologies, f_eff_achieved=1):
    """Helper function to assing same achieved efficiency

    Arguments
    ----------
    technologies : dict
        Technologies
    f_eff_achieved : float,default=1
        Factor showing the fraction of how much an efficiency is achieved

    Returns
    -------
    technologies : dict
        Adapted technolog
    """
    for technology in technologies:
        technologies[technology].eff_achieved = f_eff_achieved

    return technologies



"""Region Class
"""
class Region(object):
    """Region class

    Arguments
    ---------
    name : str
        Name of region
    longitude : float
        Longitude coordinate
    latitude : float
        Latitude coordinate
    region_fuel_disagg : dict
        Nested dict by region, enduse => np.array, single dimension for fuel type
    """
    def __init__(
            self,
            name,
            longitude,
            latitude,
            region_fuel_disagg
        ):
        """Constructor
        """
        self.name = name
        self.longitude = longitude
        self.latitude = latitude
        self.fuels = region_fuel_disagg



"""Tests for enduse functions
"""
import numpy as np
from energy_demand import enduse_func

from energy_demand.read_write import read_data
from energy_demand.technologies import technological_stock
from energy_demand.profiles import load_profile

def test_enduse():
    """testing"""
    out = enduse_func.Enduse(
        submodel_name='test_name',
        region='test',
        scenario_data='test',
        assumptions='test',
        load_profiles='test',
        base_yr='test',
        curr_yr='test',
        enduse='test',
        sector='test',
        fuel=np.zeros((7, 0)),
        tech_stock='test',
        heating_factor_y='test',
        cooling_factor_y='test',
        fuel_tech_p_by='test',
        criterias='test',
        strategy_vars='test',
        fueltypes_nr='test',
        fueltypes='test')

    assert out.flat_profile_crit == True

def test_assign_lp_no_techs():
    """Testing
    """
    lp_stock_obj = load_profile.LoadProfileStock("test_stock")

    # Shape
    _a = np.zeros((365, 24))
    _b = np.array(range(365))
    shape_yh = _a + _b[:, np.newaxis]
    shape_yh = shape_yh / float(np.sum(range(365)) * 24)

    lp_stock_obj.add_lp(
        unique_identifier="A123",
        technologies=['placeholder_tech'],
        enduses=['test_enduse'],
        shape_yd=np.full((365, 24), 1 / 365),
        shape_yh=shape_yh,
        sectors=['test_sector'],
        model_yeardays=list(range(365)),
        shape_y_dh=False)

    fuel_y = np.zeros((3))
    fuel_y[2] = 100
    fuel_yh = enduse_func.assign_lp_no_techs(
        enduse="test_enduse",
        sector="test_sector",
        load_profiles=lp_stock_obj,
        fuel_y=fuel_y,
        make_all_flat=False)

    assert np.sum(fuel_yh) == 100

'''def test_get_crit_switch():
    """
    """
    mode_constrained = True

    fuelswitches = [read_data.FuelSwitch(
        enduse='heating')]

    result = enduse_func.get_crit_switch(
        'heating',
        None,
        fuelswitches,
        2015,
        2020,
        mode_constrained)

    assert result == True

    # -----
    mode_constrained = False

    result2 = enduse_func.get_crit_switch(
        'heating',
        None,
        fuelswitches,
        2015,
        2020,
        mode_constrained)

    assert result2 == False'''

def test_get_peak_day():
    """testing
    """

    fuel_yh = np.zeros((8, 365, 24))
    fuel_yh[2][33] = 3

    result = enduse_func.get_peak_day_all_fueltypes(fuel_yh)

    expected = 33

    assert result == expected

def test_apply_service_switch():
    """Test
    """
    share_boilerA_cy = 0.2
    share_boilerB_cy = 0.8

    base_yr = 2020
    curr_yr = 2050
    end_yr = 2050

    tot_s_yh_cy = {
        'boilerA': 365*24,
        'boilerB': 365*24, }#constant share of 1 in every hour

    all_technologies = ['boilerA', 'boilerB']

    annual_tech_diff_params = {
        'boilerA': {curr_yr: share_boilerA_cy},
        'boilerB': {curr_yr: share_boilerB_cy}}

    crit_switch_happening = {'test_enduse': ['test_sector']}

    result = enduse_func.apply_service_switch(
        enduse='test_enduse',
        s_tech_y_cy=tot_s_yh_cy,
        all_technologies=all_technologies,
        curr_yr=curr_yr,
        base_yr=base_yr,
        sector='test_sector',
        annual_tech_diff_params=annual_tech_diff_params,
        crit_switch_happening=crit_switch_happening)

    expected_tech_service_cy_p = {
        "boilerA": 365*24*2 * share_boilerA_cy,
        "boilerB": 365*24*2 * share_boilerB_cy}

    assert round(expected_tech_service_cy_p["boilerA"], 3) == round(np.sum(result["boilerA"]), 3)
    assert round(expected_tech_service_cy_p["boilerB"], 3) == round(np.sum(result["boilerB"]), 3)

    # --------------------------------------------

    share_boilerA_cy = 0.99
    share_boilerB_cy = 0.01

    annual_tech_diff_params = {
        'boilerA': {curr_yr: share_boilerA_cy},
        'boilerB': {curr_yr: share_boilerB_cy}}

    result = enduse_func.apply_service_switch(
        enduse='test_enduse',
        s_tech_y_cy=tot_s_yh_cy,
        all_technologies=all_technologies,
        curr_yr=curr_yr,
        annual_tech_diff_params=annual_tech_diff_params,
        base_yr=base_yr,
        sector='test_sector',
        crit_switch_happening=crit_switch_happening)

    half_time_factor = 1
    expected_tech_service_cy_p = {
        "boilerA": (365*24*2) * share_boilerA_cy,
        "boilerB": (365*24*2) * share_boilerB_cy}

    assert round(expected_tech_service_cy_p["boilerA"], 3) == round(np.sum(result["boilerA"]), 3)
    assert round(expected_tech_service_cy_p["boilerB"], 3) == round(np.sum(result["boilerB"]), 3)

def test_convert_service_to_p():
    """Testing
    """
    tot_s_y = 8760
    s_fueltype_tech = {
        0: {
            'techA': 8760 * 0.5,
            'techB': 8760 * 0.5}
        }

    expected = enduse_func.convert_service_to_p(tot_s_y, s_fueltype_tech)

    assert expected['techA'] == 0.5
    assert expected['techB'] == 0.5

def test_calc_lf_improvement():
    """
    """
    param_lf_improved_cy = 0.25

    #all factors must be smaller than one
    loadfactor_yd_cy = np.zeros((2, 2)) #to fueltypes, two days
    loadfactor_yd_cy[0][0] = 0.2
    loadfactor_yd_cy[0][1] = 0.4
    loadfactor_yd_cy[1][0] = 0.1
    loadfactor_yd_cy[1][1] = 0.3

    result = enduse_func.calc_lf_improvement(
        param_lf_improved_cy=param_lf_improved_cy,
        loadfactor_yd_cy=loadfactor_yd_cy)

    expected = loadfactor_yd_cy + param_lf_improved_cy

    assert result[0][0] == expected[0][0]
    assert result[0][1] == expected[0][1]
    assert result[1][0] == expected[1][0]
    assert result[1][1] == expected[1][1]

def test_get_enduse_tech():
    """Testing
    """
    fuel_tech_p_by = {
        0: {'techA': 0.4, 'techB': 0.6},
        1: {'techC': 0.4, 'techD': 0.6}}
    result = enduse_func.get_enduse_techs(
        fuel_tech_p_by,
        )

    expected = ['techA', 'techB', 'techC', 'techD']
    assert expected[0] in result
    assert expected[1] in result
    assert expected[2] in result
    assert expected[3] in result

    fuel_tech_p_by = {
        0: {'techA': 0.4, 'techB': 0.6},
        1: {'placeholder_tech': 0.4, 'techD': 0.6}}
    result = enduse_func.get_enduse_techs(fuel_tech_p_by)
    expected = []
    assert expected == result

def test_apply_smart_metering():
    """Testing"""

    sm_assump = {
        'savings_smart_meter': {
            'heating': 0.5}, #50 is saved by smart meter
        'smart_meter_p_by': 0}

    strategy_vars = {}
    strategy_vars['smart_meter_p'] = {2015: 0, 2020: 0.5}

    result = enduse_func.apply_smart_metering(
        enduse='heating',
        fuel_y=100,
        sm_assump=sm_assump,
        strategy_vars=strategy_vars,
        curr_yr=2020)

    assert result == 75

def test_fuel_to_service():
    """
    """
    enduse = 'heating'
    sector = 'sectorA'
    fuel_y = {0: 2000}

    fuel_tech_p_by = {0 : {'techA': 1.0}}
    technologies = {'techA': read_data.TechnologyData()}
    technologies['techA'].fueltype_str = 'gas'
    technologies['techA'].eff_achieved = 1.0
    technologies['techA'].diff_method = 'linear'
    technologies['techA'].eff_by = 0.5
    technologies['techA'].eff_ey = 0.5
    technologies['techA'].year_eff_ey = 2020

    fueltypes = {'electricity': 0, 'gas': 1} #, 'heat': 1}

    tech_stock = technological_stock.TechStock(
        name="name",
        technologies=technologies,
        base_yr=2015,
        curr_yr=2020,
        fueltypes=fueltypes,
        temp_by=np.ones((365, 24)) + 10,
        temp_cy=np.ones((365, 24)) + 10,
        t_base_heating_by=15.5,
        potential_enduses=['heating'],
        t_base_heating_cy=15.5,
        enduse_technologies={'heating': {'sectorA': ['techA']}})

    tot_s_y, service_tech = enduse_func.fuel_to_service(
        enduse=enduse,
        sector=sector,
        fuel_y=fuel_y,
        fuel_tech_p_by=fuel_tech_p_by,
        tech_stock=tech_stock,
        fueltypes=fueltypes,
        mode_constrained=True)

    assert service_tech['techA'] == 1000

    # ---
    fuel_y = {0: 0, 1: 2000}
    fuel_tech_p_by =  { 0 : {}, 1: {'techA': 1.0}}
    fueltypes = {'gas': 0, 'heat': 1}

    tech_stock = technological_stock.TechStock(
        name="name",
        technologies=technologies,
        base_yr=2015,
        curr_yr=2020,
        fueltypes=fueltypes,
        temp_by=np.ones((365, 24)) + 10,
        temp_cy=np.ones((365, 24)) + 10,
        t_base_heating_by=15.5,
        potential_enduses=['heating'],
        t_base_heating_cy=15.5,
        enduse_technologies= {'heating': {'sectorA': ['techA']}})

    tot_s_y, service_tech = enduse_func.fuel_to_service(
        enduse=enduse,
        sector=sector,
        fuel_y=fuel_y,
        fuel_tech_p_by=fuel_tech_p_by,
        tech_stock=tech_stock,
        fueltypes=fueltypes,
        mode_constrained=False) #Difference

    assert service_tech['techA'] == 2000

def test_service_to_fuel():
    """Testing"""
    technologies = {'techA': read_data.TechnologyData()}
    technologies['techA'].fueltype_str = 'gas'
    technologies['techA'].eff_achieved = 1.0
    technologies['techA'].diff_method = 'linear'
    technologies['techA'].eff_by = 0.5
    technologies['techA'].eff_ey = 0.5
    technologies['techA'].year_eff_ey = 2020

    fueltypes = {'electricity': 0, 'gas': 1}

    tech_stock = technological_stock.TechStock(
        name="name",
        technologies=technologies,
        base_yr=2015,
        curr_yr=2020,
        fueltypes=fueltypes,
        temp_by=np.ones((365, 24)) + 10,
        temp_cy=np.ones((365, 24)) + 10,
        t_base_heating_by=15.5,
        potential_enduses=['heating'],
        t_base_heating_cy=15.5,
        enduse_technologies={'heating': {'sectorA': ['techA']}})

    fuel_y, fuel_per_tech = enduse_func.service_to_fuel(
        "heating",
        'sectorA',
        {'techA': 100},
        tech_stock,
        len(fueltypes),
        fueltypes,
        True)

    assert fuel_per_tech['techA'] == 200
    assert fuel_y[0] == 0
    assert fuel_y[1] == 200
    # ----

    fueltypes = {'gas': 0, 'heat': 1}

    fuel_y, fuel_per_tech = enduse_func.service_to_fuel(
        "heating",
        'sectorA',
        {'techA': 100},
        tech_stock,
        len(fueltypes),
        fueltypes,
        False)

    assert fuel_per_tech['techA'] == 100
    assert fuel_y[1] == 100

def test_apply_heat_recovery():
    """Testing"""

    heat_recovered = {'heating': {2015: 0, 2020: 0.5}}
    result, result_tech = enduse_func.apply_heat_recovery(
        enduse='heating',
        heat_recovered=heat_recovered,
        service=100,
        service_techs={'techA': 100},
        curr_yr=2020)

    assert result == 50
    assert result_tech == {'techA': 50}

def test_apply_weather_correction():
    """Testing"""

    result = enduse_func.apply_weather_correction(
        enduse='heating',
        fuel_y=200,
        cooling_factor_y=1.5,
        heating_factor_y=1.5,
        enduse_space_heating=['heating'],
        enduse_space_cooling=['cooling'])

    assert result == 300
    result = enduse_func.apply_weather_correction(
        enduse='cooling',
        fuel_y=200,
        cooling_factor_y=1.5,
        heating_factor_y=1.5,
        enduse_space_heating=['heating'],
        enduse_space_cooling=['cooling'])

    assert result == 300

def test_calc_fuel_tech_yh():
    """Testing
    """
    fueltypes = {'gas': 0, 'heat': 1}

    technologies = {'techA': read_data.TechnologyData()}
    technologies['techA'].fueltype_str = 'gas'
    technologies['techA'].eff_achieved = 1.0
    technologies['techA'].diff_method = 'linear'
    technologies['techA'].eff_by = 0.5
    technologies['techA'].eff_ey = 0.5
    technologies['techA'].year_eff_ey = 2020

    '''tech_stock = technological_stock.TechStock(
        name="name",
        technologies=technologies,
        base_yr=2015,
        curr_yr=2020,
        fueltypes=fueltypes,
        temp_by=np.ones((365, 24)) + 10,
        temp_cy=np.ones((365, 24)) + 10,
        t_base_heating_by=15.5,
        potential_enduses=['heating'],
        t_base_heating_cy=15.5,
        enduse_technologies={'heating': ['techA']})'''

    lp_stock_obj = load_profile.LoadProfileStock("test_stock")

    _a = np.zeros((365, 24))
    _b = np.array(range(365))
    shape_yh = _a + _b[:, np.newaxis]
    shape_yh = shape_yh / float(np.sum(range(365)) * 24)
    model_yeardays = list(range(365))

    lp_stock_obj.add_lp(
        unique_identifier="A123",
        technologies=['techA'],
        enduses=['heating'],
        shape_yd=np.full((365,24), 1 / 365),
        shape_yh=shape_yh,
        sectors=['sectorA'],
        shape_y_dh=np.full((365), 1/365),
        model_yeardays=model_yeardays)

    fuel = 200
    results = enduse_func.calc_fuel_tech_yh(
        enduse='heating',
        sector='sectorA',
        enduse_techs=['techA'],
        fuel_tech_y={'techA': fuel},
        load_profiles=lp_stock_obj,
        fueltypes_nr=2,
        fueltypes=fueltypes,
        mode_constrained=False)

    assert results[1][3][0] == 3.0 / float(np.sum(range(365)) * 24) * 200

    # ---
    results = enduse_func.calc_fuel_tech_yh(
        enduse='heating',
        sector='sectorA',
        enduse_techs=['techA'],
        fuel_tech_y={'techA': fuel},
        load_profiles=lp_stock_obj,
        fueltypes_nr=2,
        fueltypes=fueltypes,
        mode_constrained=True)

    assert results['techA'][3][0] == 3.0 / float(np.sum(range(365)) * 24) * 200

def test_generic_demand_change():
    """testing
    """
    generic_enduse_change = {'heating': {2015: 0, 2020: 2.0}}
    generic_enduse_change['heating']['param_info'] = {}
    generic_enduse_change['heating']['param_info']['sector'] = 'heating'

    fuel_y = np.array([100])
    result = enduse_func.generic_demand_change(
        sector=True,
        enduse='heating',
        fuel_y=fuel_y,
        generic_enduse_change=generic_enduse_change,
        curr_yr=2020)

    assert result == fuel_y * (1 + generic_enduse_change['heating'][2020])

def test_get_enduse_configuration():
    """Testing
    """
    fuel_switches = [read_data.FuelSwitch(
        enduse='heating',
        fueltype_replace="",
        technology_install='boilerB',
        switch_yr=2020,
        fuel_share_switched_ey="")]

    service_switches = [read_data.ServiceSwitch(
        technology_install='boilerA',
        switch_yr=2050)]

    mode_constrained = enduse_func.get_enduse_configuration(
        mode_constrained=False,
        enduse='heating',
        enduse_space_heating=['heating'])

    assert mode_constrained == False

    # ---
    service_switches = [read_data.ServiceSwitch(
        enduse='heating',
        technology_install='boilerA',
        switch_yr=2050)]

    mode_constrained = enduse_func.get_enduse_configuration(
        mode_constrained=True,
        enduse='heating',
        enduse_space_heating=['heating'])

    assert mode_constrained == True

    # ---
    fuel_switches = [read_data.FuelSwitch(
        enduse='heating',
        fueltype_replace="",
        technology_install='boilerB',
        switch_yr=2020,
        fuel_share_switched_ey=""
    )]

    service_switches = []

    mode_constrained = enduse_func.get_enduse_configuration(
        mode_constrained=False,
        enduse='heating',
        enduse_space_heating=['heating'])

    assert mode_constrained == False

def test_apply_cooling():
    """testing
    """
    cooled_floorarea = {'cooling_enduse': {2015: 0, 2020: 0.5}}

    assump_cooling_floorarea = 0.25
    fuel_y = np.array([100])

    result = enduse_func.apply_cooling(
        enduse='cooling_enduse',
        fuel_y=fuel_y,
        cooled_floorarea=cooled_floorarea,
        cooled_floorarea_p_by=assump_cooling_floorarea,
        curr_yr=2020)

    assert np.sum(result) == np.sum(fuel_y) * cooled_floorarea['cooling_enduse'][2020] / assump_cooling_floorarea



"""
Create dynamic gif file from results

VALID_EXTENSIONS = ('png', 'jpg')

http://www.idiotinside.com/2017/06/06/create-gif-animation-with-python/

"""
import os
import datetime
import imageio

def create_gif(filenames, duration, path):
    """
    """
    images = []
    for filename in filenames:
        images.append(imageio.imread(filename))
    output_file_name = 'Gif-%s.gif' % datetime.datetime.now().strftime('%Y-%M-%d-%H-%M-%S')
    output_file = os.path.join(path, output_file_name)
    imageio.mimsave(output_file, images, duration=duration)


'''if __name__ == "__main__":
    script = sys.argv.pop(0)

    if len(sys.argv) < 2:
        print('Usage: python {} <duration> <path to images separated by space>'.format(script))
        sys.exit(1)

    duration = float(sys.argv.pop(0))
    filenames = sys.argv


    if not all(f.lower().endswith(VALID_EXTENSIONS) for f in filenames):
        print('Only png and jpg files allowed')
        sys.exit(1)

    create_gif(filenames, duration)'''


def get_all_results(path_to_scenario, file_name_part):

    
    scenario_results = os.path.join(path_to_scenario, '_result_data', 'spatial_results')

    # Iterate all files
    result_files = os.listdir(scenario_results)

    filenames = []

    for file_name in result_files:

        try:
            print(file_name.split(file_name_part))
            if len(file_name.split(file_name_part)) == 2:
                filenames.append(os.path.join(scenario_results, file_name))
        except:
            pass
    
    create_gif(filenames, duration=1, path=scenario_results)
    print("finished creating gif")

#get_all_results(r"C:\Users\cenv0553\nismod\data_energy_demand\_MULT2\ase", 'lf_diff')


"""
"""
from pytest import fixture
from unittest.mock import Mock
from energy_demand.read_write.data_loader import get_local_paths


@fixture(scope='function')
def config_file(tmpdir):

    config_file = tmpdir.mkdir("config").join("wrapperconfig.ini")
    content = \
"""
[PATHS]
path_local_data = /vagrant/data/energy_demand
path_processed_data = /vagrant/data/energy_demand/_processed_data
path_result_data = /vagrant/data/energy_demand/results
path_config_data = /vagrant/data/energy_demand/config_data

[CONFIG]
base_yr = 2015
weather_yr_scenario = 2015
user_defined_simulation_end_yr = 2050
user_defined_weather_by = 2015

[CRITERIA]
mode_constrained = True
virtual_building_stock_criteria = True
write_out_national = False
reg_selection = False
MSOA_crit = False
reg_selection_csv_name = msoa_regions_ed.csv
spatial_calibration = False
cluster_calc = False
write_txt_additional_results = True
validation_criteria = True
plot_crit = False
crit_plot_enduse_lp = False
writeYAML_keynames = True
writeYAML = False
crit_temp_min_max = False

[DATA_PATHS]
local_path_datafolder = /vagrant/data/energy_demand
path_strategy_vars =  /vagrant/data/energy_demand/energy_demand_minimal/00_user_defined_variables
#ONS principal projection
path_population_data_for_disaggregation_LAD =  /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/J-population_disagg_by/uk_pop_principal_2015_2050.csv
#ONS principal projection
folder_raw_carbon_trust = /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/G_Carbon_Trust_advanced_metering_trial/
path_population_data_for_disaggregation_MSOA = /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/J-population_disagg_by/uk_pop_principal_2015_2050_MSOA_lad.csv
folder_path_weater_stations = /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/A-temperature_data/cleaned_weather_stations.csv
path_floor_area_virtual_stock_by = /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/K-floor_area/floor_area_LAD_latest.csv
path_assumptions_db = /vagrant/data/energy_demand/_processed_data/assumptions_from_db
data_processed = /vagrant/data/energy_demand/_processed_data
lad_shapefile = /vagrant/data/energy_demand/energy_demand_minimal/_raw_data/C_LAD_geography/same_as_pop_scenario/lad_2016_uk_simplified.shp
path_post_installation_data = /vagrant/data/energy_demand/_processed_data
weather_data = _/raw_data/A-temperature_data/cleaned_weather_stations_data
load_profiles = /vagrant/data/energy_demand/_processed_data/load_profiles
rs_load_profile_txt = /vagrant/data/energy_demand/_processed_data/load_profiles/rs_submodel),
ss_load_profile_txt = /vagrant/data/energy_demand/_processed_data/load_profiles/ss_submodel),
yaml_parameters = /vagrant/data/energy_demand/config/yaml_parameters.yml
yaml_parameters_constrained = /vagrant/data/energy_demand/config/yaml_parameters_constrained.yml
yaml_parameters_keynames_constrained = /vagrant/data/energy_demand/config/yaml_parameters_keynames_constrained.yml
yaml_parameters_keynames_unconstrained = /vagrant/data/energy_demand/config/yaml_parameters_keynames_unconstrained.yml
yaml_parameters_scenario = /vagrant/data/energy_demand/config/yaml_parameters_scenario.yml

"""

    config_file.write(content)

    return str(config_file)


def test_get_local_paths(config_file):

    actual = get_local_paths(config_file)

    assert isinstance(actual, dict)
    assert actual['yaml_parameters_scenario'] == '/vagrant/data/energy_demand/config/yaml_parameters_scenario.yml'
